// =============================================================================
// == linear_algebra.h
// == --------------------------------------------------------------------------
// == Some functions to help quickly calculate things.  Includes linear algebra
// == functions, and some others that should probably be moved somewhere else
// ==
// == det : determinant
// == xt_A_x : vector matrix vector product
// == xt_Ainv_x : vector matrix^-1 vector product
// == InvMat2x2 : inverse of 2D matrix
// == my_log : fast table look-up log in 0.5 increments (0.5, 1, 1.5, ...)
// == my_sf_lngamma : log(gamma(x/2)) for integer x's
// == mylogmgamma : multivariate log(gamma(x/2)) for integer x's
// ==
// == All work using this code should cite:
// == J. Chang, D. Wei, and J. W. Fisher III. A Video Representation Using
// ==    Temporal Superpixels. CVPR 2013.
// == --------------------------------------------------------------------------
// == Written by Jason Chang and Donglai Wei 06-20-2013
// =============================================================================

#ifndef LINEAR_ALGEBRA_H
#define LINEAR_ALGEBRA_H

#include "helperMEX.h"
#include <math.h>
#include "gsl/gsl_sf_gamma.h"

#ifndef pi
#define pi 3.14159265
#endif

inline double det(arr(double) mtx, int D)
{
   double determinant;
   if (D==1)
      determinant = mtx[0];
   else if (D==2)
      determinant = mtx[0]*mtx[3] - mtx[1]*mtx[2];
   else if (D==3)
   {
      // 0 3 6
      // 1 4 7
      // 2 5 8

      determinant = mtx[0]*(mtx[4]*mtx[8]-mtx[5]*mtx[7])
                  - mtx[3]*(mtx[1]*mtx[8]-mtx[2]*mtx[7])
                  + mtx[6]*(mtx[1]*mtx[5]-mtx[2]*mtx[4]);
   }
   else
   {
      mexErrMsgTxt("D>3 Not supported for determinants\n");
   }
   return determinant;
}

// computes x'Ax, where x is Dx1, and A is DxD symmetric matrix
inline double xt_A_x(arr(double) x, arr(double)A, int D)
{
   double val, x0, x1, x2, A00, A01, A02, A11, A12, A22;
   switch (D)
   {
      case 1:
         mexErrMsgTxt("xt_A_x not supported!");
         break;
      case 2:
         x0 = x[0];
         x1 = x[1];
         A00 = A[0];
         A01 = A[1];
         A11 = A[3];
         val = x0*(x0*A00 + x1*A01) + x1*(x0*A01 + x1*A11);
         break;
      case 3:
         x0 = x[0];
         x1 = x[1];
         x2 = x[2];
         A00 = A[0];
         A01 = A[1];
         A02 = A[2];
         A11 = A[4];
         A12 = A[5];
         A22 = A[8];
         val = x0*(x0*A00 + x1*A01 + x2*A02) + x1*(x0*A01 + x1*A11 + x2*A12) + x2*(x0*A02 + x1*A12 + x2*A22);
         break;
      default:
         mexErrMsgTxt("xt_A_x not supported!");
         break;
   }
   return val;
}
// computes x'Ax, where x is Dx1, and A is DxD symmetric matrix
inline double xt_Ainv_x(arr(double) x, arr(double)A, int D)
{
   double determinant = det(A, D);
   double val, x0, x1, x2, A00, A01, A02, A11, A12, A22, a00, a01, a02, a11, a12, a22;
   switch (D)
   {
      case 1:
         mexErrMsgTxt("xt_A_x not supported!");
         break;
      case 2:
         x0 = x[0];
         x1 = x[1];
         A00 = A[3]/determinant;
         A01 = -A[1]/determinant;
         A11 = A[0]/determinant;
         val = x0*(x0*A00 + x1*A01) + x1*(x0*A01 + x1*A11);
         break;
      case 3:
         x0 = x[0];
         x1 = x[1];
         x2 = x[2];
         a00 = A[0];
         a01 = A[1];
         a02 = A[2];
         a11 = A[4];
         a12 = A[5];
         a22 = A[8];

         A00 = (a22*a11 - a12*a12)/determinant;
         A01 = (a02*a01 - a22*a01)/determinant;
         A02 = (a12*a01 - a02*a11)/determinant;
         A11 = (a00*a22 - a02*a02)/determinant;
         A12 = (a02*a01 - a12*a00)/determinant;
         A22 = (a11*a00 - a01*a01)/determinant;

         val = x0*(x0*A00 + x1*A01 + x2*A02) + x1*(x0*A01 + x1*A11 + x2*A12) + x2*(x0*A02 + x1*A12 + x2*A22);
         break;
      default:
         mexErrMsgTxt("xt_A_x not supported!");
         break;
   }
   return val;
}

inline void InvMat2x2(double *mat1, double *result){

    double tmp_sum = (mat1[0]*mat1[3]-mat1[1]*mat1[2]);
    if(tmp_sum<=0 || fabs(mat1[1]-mat1[2])>1E-5)
       mexErrMsgTxt("Input matrix should be positive definite! ");

    tmp_sum = 1/tmp_sum;
    result[0] = mat1[3] * tmp_sum;
    result[1] = -mat1[1] * tmp_sum;
    result[2] = result[1];
    result[3] = mat1[0] * tmp_sum;
}



inline double my_log(double x)
{
   static const double temp_log[4000] = {
   -1E10, 0.000000, 0.693147, 1.098612, 1.386294, 1.609438, 1.791759, 1.945910, 2.079442, 2.197225,
2.302585, 2.397895, 2.484907, 2.564949, 2.639057, 2.708050, 2.772589, 2.833213, 2.890372, 2.944439,
2.995732, 3.044522, 3.091042, 3.135494, 3.178054, 3.218876, 3.258097, 3.295837, 3.332205, 3.367296,
3.401197, 3.433987, 3.465736, 3.496508, 3.526361, 3.555348, 3.583519, 3.610918, 3.637586, 3.663562,
3.688879, 3.713572, 3.737670, 3.761200, 3.784190, 3.806662, 3.828641, 3.850148, 3.871201, 3.891820,
3.912023, 3.931826, 3.951244, 3.970292, 3.988984, 4.007333, 4.025352, 4.043051, 4.060443, 4.077537,
4.094345, 4.110874, 4.127134, 4.143135, 4.158883, 4.174387, 4.189655, 4.204693, 4.219508, 4.234107,
4.248495, 4.262680, 4.276666, 4.290459, 4.304065, 4.317488, 4.330733, 4.343805, 4.356709, 4.369448,
4.382027, 4.394449, 4.406719, 4.418841, 4.430817, 4.442651, 4.454347, 4.465908, 4.477337, 4.488636,
4.499810, 4.510860, 4.521789, 4.532599, 4.543295, 4.553877, 4.564348, 4.574711, 4.584967, 4.595120,
4.605170, 4.615121, 4.624973, 4.634729, 4.644391, 4.653960, 4.663439, 4.672829, 4.682131, 4.691348,
4.700480, 4.709530, 4.718499, 4.727388, 4.736198, 4.744932, 4.753590, 4.762174, 4.770685, 4.779123,
4.787492, 4.795791, 4.804021, 4.812184, 4.820282, 4.828314, 4.836282, 4.844187, 4.852030, 4.859812,
4.867534, 4.875197, 4.882802, 4.890349, 4.897840, 4.905275, 4.912655, 4.919981, 4.927254, 4.934474,
4.941642, 4.948760, 4.955827, 4.962845, 4.969813, 4.976734, 4.983607, 4.990433, 4.997212, 5.003946,
5.010635, 5.017280, 5.023881, 5.030438, 5.036953, 5.043425, 5.049856, 5.056246, 5.062595, 5.068904,
5.075174, 5.081404, 5.087596, 5.093750, 5.099866, 5.105945, 5.111988, 5.117994, 5.123964, 5.129899,
5.135798, 5.141664, 5.147494, 5.153292, 5.159055, 5.164786, 5.170484, 5.176150, 5.181784, 5.187386,
5.192957, 5.198497, 5.204007, 5.209486, 5.214936, 5.220356, 5.225747, 5.231109, 5.236442, 5.241747,
5.247024, 5.252273, 5.257495, 5.262690, 5.267858, 5.273000, 5.278115, 5.283204, 5.288267, 5.293305,
5.298317, 5.303305, 5.308268, 5.313206, 5.318120, 5.323010, 5.327876, 5.332719, 5.337538, 5.342334,
5.347108, 5.351858, 5.356586, 5.361292, 5.365976, 5.370638, 5.375278, 5.379897, 5.384495, 5.389072,
5.393628, 5.398163, 5.402677, 5.407172, 5.411646, 5.416100, 5.420535, 5.424950, 5.429346, 5.433722,
5.438079, 5.442418, 5.446737, 5.451038, 5.455321, 5.459586, 5.463832, 5.468060, 5.472271, 5.476464,
5.480639, 5.484797, 5.488938, 5.493061, 5.497168, 5.501258, 5.505332, 5.509388, 5.513429, 5.517453,
5.521461, 5.525453, 5.529429, 5.533389, 5.537334, 5.541264, 5.545177, 5.549076, 5.552960, 5.556828,
5.560682, 5.564520, 5.568345, 5.572154, 5.575949, 5.579730, 5.583496, 5.587249, 5.590987, 5.594711,
5.598422, 5.602119, 5.605802, 5.609472, 5.613128, 5.616771, 5.620401, 5.624018, 5.627621, 5.631212,
5.634790, 5.638355, 5.641907, 5.645447, 5.648974, 5.652489, 5.655992, 5.659482, 5.662960, 5.666427,
5.669881, 5.673323, 5.676754, 5.680173, 5.683580, 5.686975, 5.690359, 5.693732, 5.697093, 5.700444,
5.703782, 5.707110, 5.710427, 5.713733, 5.717028, 5.720312, 5.723585, 5.726848, 5.730100, 5.733341,
5.736572, 5.739793, 5.743003, 5.746203, 5.749393, 5.752573, 5.755742, 5.758902, 5.762051, 5.765191,
5.768321, 5.771441, 5.774552, 5.777652, 5.780744, 5.783825, 5.786897, 5.789960, 5.793014, 5.796058,
5.799093, 5.802118, 5.805135, 5.808142, 5.811141, 5.814131, 5.817111, 5.820083, 5.823046, 5.826000,
5.828946, 5.831882, 5.834811, 5.837730, 5.840642, 5.843544, 5.846439, 5.849325, 5.852202, 5.855072,
5.857933, 5.860786, 5.863631, 5.866468, 5.869297, 5.872118, 5.874931, 5.877736, 5.880533, 5.883322,
5.886104, 5.888878, 5.891644, 5.894403, 5.897154, 5.899897, 5.902633, 5.905362, 5.908083, 5.910797,
5.913503, 5.916202, 5.918894, 5.921578, 5.924256, 5.926926, 5.929589, 5.932245, 5.934894, 5.937536,
5.940171, 5.942799, 5.945421, 5.948035, 5.950643, 5.953243, 5.955837, 5.958425, 5.961005, 5.963579,
5.966147, 5.968708, 5.971262, 5.973810, 5.976351, 5.978886, 5.981414, 5.983936, 5.986452, 5.988961,
5.991465, 5.993961, 5.996452, 5.998937, 6.001415, 6.003887, 6.006353, 6.008813, 6.011267, 6.013715,
6.016157, 6.018593, 6.021023, 6.023448, 6.025866, 6.028279, 6.030685, 6.033086, 6.035481, 6.037871,
6.040255, 6.042633, 6.045005, 6.047372, 6.049733, 6.052089, 6.054439, 6.056784, 6.059123, 6.061457,
6.063785, 6.066108, 6.068426, 6.070738, 6.073045, 6.075346, 6.077642, 6.079933, 6.082219, 6.084499,
6.086775, 6.089045, 6.091310, 6.093570, 6.095825, 6.098074, 6.100319, 6.102559, 6.104793, 6.107023,
6.109248, 6.111467, 6.113682, 6.115892, 6.118097, 6.120297, 6.122493, 6.124683, 6.126869, 6.129050,
6.131226, 6.133398, 6.135565, 6.137727, 6.139885, 6.142037, 6.144186, 6.146329, 6.148468, 6.150603,
6.152733, 6.154858, 6.156979, 6.159095, 6.161207, 6.163315, 6.165418, 6.167516, 6.169611, 6.171701,
6.173786, 6.175867, 6.177944, 6.180017, 6.182085, 6.184149, 6.186209, 6.188264, 6.190315, 6.192362,
6.194405, 6.196444, 6.198479, 6.200509, 6.202536, 6.204558, 6.206576, 6.208590, 6.210600, 6.212606,
6.214608, 6.216606, 6.218600, 6.220590, 6.222576, 6.224558, 6.226537, 6.228511, 6.230481, 6.232448,
6.234411, 6.236370, 6.238325, 6.240276, 6.242223, 6.244167, 6.246107, 6.248043, 6.249975, 6.251904,
6.253829, 6.255750, 6.257668, 6.259581, 6.261492, 6.263398, 6.265301, 6.267201, 6.269096, 6.270988,
6.272877, 6.274762, 6.276643, 6.278521, 6.280396, 6.282267, 6.284134, 6.285998, 6.287859, 6.289716,
6.291569, 6.293419, 6.295266, 6.297109, 6.298949, 6.300786, 6.302619, 6.304449, 6.306275, 6.308098,
6.309918, 6.311735, 6.313548, 6.315358, 6.317165, 6.318968, 6.320768, 6.322565, 6.324359, 6.326149,
6.327937, 6.329721, 6.331502, 6.333280, 6.335054, 6.336826, 6.338594, 6.340359, 6.342121, 6.343880,
6.345636, 6.347389, 6.349139, 6.350886, 6.352629, 6.354370, 6.356108, 6.357842, 6.359574, 6.361302,
6.363028, 6.364751, 6.366470, 6.368187, 6.369901, 6.371612, 6.373320, 6.375025, 6.376727, 6.378426,
6.380123, 6.381816, 6.383507, 6.385194, 6.386879, 6.388561, 6.390241, 6.391917, 6.393591, 6.395262,
6.396930, 6.398595, 6.400257, 6.401917, 6.403574, 6.405228, 6.406880, 6.408529, 6.410175, 6.411818,
6.413459, 6.415097, 6.416732, 6.418365, 6.419995, 6.421622, 6.423247, 6.424869, 6.426488, 6.428105,
6.429719, 6.431331, 6.432940, 6.434547, 6.436150, 6.437752, 6.439350, 6.440947, 6.442540, 6.444131,
6.445720, 6.447306, 6.448889, 6.450470, 6.452049, 6.453625, 6.455199, 6.456770, 6.458338, 6.459904,
6.461468, 6.463029, 6.464588, 6.466145, 6.467699, 6.469250, 6.470800, 6.472346, 6.473891, 6.475433,
6.476972, 6.478510, 6.480045, 6.481577, 6.483107, 6.484635, 6.486161, 6.487684, 6.489205, 6.490724,
6.492240, 6.493754, 6.495266, 6.496775, 6.498282, 6.499787, 6.501290, 6.502790, 6.504288, 6.505784,
6.507278, 6.508769, 6.510258, 6.511745, 6.513230, 6.514713, 6.516193, 6.517671, 6.519147, 6.520621,
6.522093, 6.523562, 6.525030, 6.526495, 6.527958, 6.529419, 6.530878, 6.532334, 6.533789, 6.535241,
6.536692, 6.538140, 6.539586, 6.541030, 6.542472, 6.543912, 6.545350, 6.546785, 6.548219, 6.549651,
6.551080, 6.552508, 6.553933, 6.555357, 6.556778, 6.558198, 6.559615, 6.561031, 6.562444, 6.563856,
6.565265, 6.566672, 6.568078, 6.569481, 6.570883, 6.572283, 6.573680, 6.575076, 6.576470, 6.577861,
6.579251, 6.580639, 6.582025, 6.583409, 6.584791, 6.586172, 6.587550, 6.588926, 6.590301, 6.591674,
6.593045, 6.594413, 6.595781, 6.597146, 6.598509, 6.599870, 6.601230, 6.602588, 6.603944, 6.605298,
6.606650, 6.608001, 6.609349, 6.610696, 6.612041, 6.613384, 6.614726, 6.616065, 6.617403, 6.618739,
6.620073, 6.621406, 6.622736, 6.624065, 6.625392, 6.626718, 6.628041, 6.629363, 6.630683, 6.632002,
6.633318, 6.634633, 6.635947, 6.637258, 6.638568, 6.639876, 6.641182, 6.642487, 6.643790, 6.645091,
6.646391, 6.647688, 6.648985, 6.650279, 6.651572, 6.652863, 6.654153, 6.655440, 6.656727, 6.658011,
6.659294, 6.660575, 6.661855, 6.663133, 6.664409, 6.665684, 6.666957, 6.668228, 6.669498, 6.670766,
6.672033, 6.673298, 6.674561, 6.675823, 6.677083, 6.678342, 6.679599, 6.680855, 6.682109, 6.683361,
6.684612, 6.685861, 6.687109, 6.688355, 6.689599, 6.690842, 6.692084, 6.693324, 6.694562, 6.695799,
6.697034, 6.698268, 6.699500, 6.700731, 6.701960, 6.703188, 6.704414, 6.705639, 6.706862, 6.708084,
6.709304, 6.710523, 6.711740, 6.712956, 6.714171, 6.715383, 6.716595, 6.717805, 6.719013, 6.720220,
6.721426, 6.722630, 6.723832, 6.725034, 6.726233, 6.727432, 6.728629, 6.729824, 6.731018, 6.732211,
6.733402, 6.734592, 6.735780, 6.736967, 6.738152, 6.739337, 6.740519, 6.741701, 6.742881, 6.744059,
6.745236, 6.746412, 6.747587, 6.748760, 6.749931, 6.751101, 6.752270, 6.753438, 6.754604, 6.755769,
6.756932, 6.758095, 6.759255, 6.760415, 6.761573, 6.762730, 6.763885, 6.765039, 6.766192, 6.767343,
6.768493, 6.769642, 6.770789, 6.771936, 6.773080, 6.774224, 6.775366, 6.776507, 6.777647, 6.778785,
6.779922, 6.781058, 6.782192, 6.783325, 6.784457, 6.785588, 6.786717, 6.787845, 6.788972, 6.790097,
6.791221, 6.792344, 6.793466, 6.794587, 6.795706, 6.796824, 6.797940, 6.799056, 6.800170, 6.801283,
6.802395, 6.803505, 6.804615, 6.805723, 6.806829, 6.807935, 6.809039, 6.810142, 6.811244, 6.812345,
6.813445, 6.814543, 6.815640, 6.816736, 6.817831, 6.818924, 6.820016, 6.821107, 6.822197, 6.823286,
6.824374, 6.825460, 6.826545, 6.827629, 6.828712, 6.829794, 6.830874, 6.831954, 6.833032, 6.834109,
6.835185, 6.836259, 6.837333, 6.838405, 6.839476, 6.840547, 6.841615, 6.842683, 6.843750, 6.844815,
6.845880, 6.846943, 6.848005, 6.849066, 6.850126, 6.851185, 6.852243, 6.853299, 6.854355, 6.855409,
6.856462, 6.857514, 6.858565, 6.859615, 6.860664, 6.861711, 6.862758, 6.863803, 6.864848, 6.865891,
6.866933, 6.867974, 6.869014, 6.870053, 6.871091, 6.872128, 6.873164, 6.874198, 6.875232, 6.876265,
6.877296, 6.878326, 6.879356, 6.880384, 6.881411, 6.882437, 6.883463, 6.884487, 6.885510, 6.886532,
6.887553, 6.888572, 6.889591, 6.890609, 6.891626, 6.892642, 6.893656, 6.894670, 6.895683, 6.896694,
6.897705, 6.898715, 6.899723, 6.900731, 6.901737, 6.902743, 6.903747, 6.904751, 6.905753, 6.906755,
6.907755, 6.908755, 6.909753, 6.910751, 6.911747, 6.912743, 6.913737, 6.914731, 6.915723, 6.916715,
6.917706, 6.918695, 6.919684, 6.920672, 6.921658, 6.922644, 6.923629, 6.924612, 6.925595, 6.926577,
6.927558, 6.928538, 6.929517, 6.930495, 6.931472, 6.932448, 6.933423, 6.934397, 6.935370, 6.936343,
6.937314, 6.938284, 6.939254, 6.940222, 6.941190, 6.942157, 6.943122, 6.944087, 6.945051, 6.946014,
6.946976, 6.947937, 6.948897, 6.949856, 6.950815, 6.951772, 6.952729, 6.953684, 6.954639, 6.955593,
6.956545, 6.957497, 6.958448, 6.959399, 6.960348, 6.961296, 6.962243, 6.963190, 6.964136, 6.965080,
6.966024, 6.966967, 6.967909, 6.968850, 6.969791, 6.970730, 6.971669, 6.972606, 6.973543, 6.974479,
6.975414, 6.976348, 6.977281, 6.978214, 6.979145, 6.980076, 6.981006, 6.981935, 6.982863, 6.983790,
6.984716, 6.985642, 6.986566, 6.987490, 6.988413, 6.989335, 6.990257, 6.991177, 6.992096, 6.993015,
6.993933, 6.994850, 6.995766, 6.996681, 6.997596, 6.998510, 6.999422, 7.000334, 7.001246, 7.002156,
7.003065, 7.003974, 7.004882, 7.005789, 7.006695, 7.007601, 7.008505, 7.009409, 7.010312, 7.011214,
7.012115, 7.013016, 7.013915, 7.014814, 7.015712, 7.016610, 7.017506, 7.018402, 7.019297, 7.020191,
7.021084, 7.021976, 7.022868, 7.023759, 7.024649, 7.025538, 7.026427, 7.027315, 7.028201, 7.029088,
7.029973, 7.030857, 7.031741, 7.032624, 7.033506, 7.034388, 7.035269, 7.036148, 7.037028, 7.037906,
7.038784, 7.039660, 7.040536, 7.041412, 7.042286, 7.043160, 7.044033, 7.044905, 7.045777, 7.046647,
7.047517, 7.048386, 7.049255, 7.050123, 7.050989, 7.051856, 7.052721, 7.053586, 7.054450, 7.055313,
7.056175, 7.057037, 7.057898, 7.058758, 7.059618, 7.060476, 7.061334, 7.062192, 7.063048, 7.063904,
7.064759, 7.065613, 7.066467, 7.067320, 7.068172, 7.069023, 7.069874, 7.070724, 7.071573, 7.072422,
7.073270, 7.074117, 7.074963, 7.075809, 7.076654, 7.077498, 7.078342, 7.079184, 7.080026, 7.080868,
7.081709, 7.082549, 7.083388, 7.084226, 7.085064, 7.085901, 7.086738, 7.087574, 7.088409, 7.089243,
7.090077, 7.090910, 7.091742, 7.092574, 7.093405, 7.094235, 7.095064, 7.095893, 7.096721, 7.097549,
7.098376, 7.099202, 7.100027, 7.100852, 7.101676, 7.102499, 7.103322, 7.104144, 7.104965, 7.105786,
7.106606, 7.107425, 7.108244, 7.109062, 7.109879, 7.110696, 7.111512, 7.112327, 7.113142, 7.113956,
7.114769, 7.115582, 7.116394, 7.117206, 7.118016, 7.118826, 7.119636, 7.120444, 7.121252, 7.122060,
7.122867, 7.123673, 7.124478, 7.125283, 7.126087, 7.126891, 7.127694, 7.128496, 7.129298, 7.130099,
7.130899, 7.131699, 7.132498, 7.133296, 7.134094, 7.134891, 7.135687, 7.136483, 7.137278, 7.138073,
7.138867, 7.139660, 7.140453, 7.141245, 7.142037, 7.142827, 7.143618, 7.144407, 7.145196, 7.145984,
7.146772, 7.147559, 7.148346, 7.149132, 7.149917, 7.150701, 7.151485, 7.152269, 7.153052, 7.153834,
7.154615, 7.155396, 7.156177, 7.156956, 7.157735, 7.158514, 7.159292, 7.160069, 7.160846, 7.161622,
7.162397, 7.163172, 7.163947, 7.164720, 7.165493, 7.166266, 7.167038, 7.167809, 7.168580, 7.169350,
7.170120, 7.170888, 7.171657, 7.172425, 7.173192, 7.173958, 7.174724, 7.175490, 7.176255, 7.177019,
7.177782, 7.178545, 7.179308, 7.180070, 7.180831, 7.181592, 7.182352, 7.183112, 7.183871, 7.184629,
7.185387, 7.186144, 7.186901, 7.187657, 7.188413, 7.189168, 7.189922, 7.190676, 7.191429, 7.192182,
7.192934, 7.193686, 7.194437, 7.195187, 7.195937, 7.196687, 7.197435, 7.198184, 7.198931, 7.199678,
7.200425, 7.201171, 7.201916, 7.202661, 7.203406, 7.204149, 7.204893, 7.205635, 7.206377, 7.207119,
7.207860, 7.208600, 7.209340, 7.210080, 7.210818, 7.211557, 7.212294, 7.213032, 7.213768, 7.214504,
7.215240, 7.215975, 7.216709, 7.217443, 7.218177, 7.218910, 7.219642, 7.220374, 7.221105, 7.221836,
7.222566, 7.223296, 7.224025, 7.224753, 7.225481, 7.226209, 7.226936, 7.227662, 7.228388, 7.229114,
7.229839, 7.230563, 7.231287, 7.232010, 7.232733, 7.233455, 7.234177, 7.234898, 7.235619, 7.236339,
7.237059, 7.237778, 7.238497, 7.239215, 7.239933, 7.240650, 7.241366, 7.242082, 7.242798, 7.243513,
7.244228, 7.244942, 7.245655, 7.246368, 7.247081, 7.247793, 7.248504, 7.249215, 7.249926, 7.250636,
7.251345, 7.252054, 7.252762, 7.253470, 7.254178, 7.254885, 7.255591, 7.256297, 7.257003, 7.257708,
7.258412, 7.259116, 7.259820, 7.260523, 7.261225, 7.261927, 7.262629, 7.263330, 7.264030, 7.264730,
7.265430, 7.266129, 7.266827, 7.267525, 7.268223, 7.268920, 7.269617, 7.270313, 7.271009, 7.271704,
7.272398, 7.273093, 7.273786, 7.274480, 7.275172, 7.275865, 7.276556, 7.277248, 7.277939, 7.278629,
7.279319, 7.280008, 7.280697, 7.281386, 7.282074, 7.282761, 7.283448, 7.284135, 7.284821, 7.285507,
7.286192, 7.286876, 7.287561, 7.288244, 7.288928, 7.289611, 7.290293, 7.290975, 7.291656, 7.292337,
7.293018, 7.293698, 7.294377, 7.295056, 7.295735, 7.296413, 7.297091, 7.297768, 7.298445, 7.299121,
7.299797, 7.300473, 7.301148, 7.301822, 7.302496, 7.303170, 7.303843, 7.304516, 7.305188, 7.305860,
7.306531, 7.307202, 7.307873, 7.308543, 7.309212, 7.309881, 7.310550, 7.311218, 7.311886, 7.312553,
7.313220, 7.313887, 7.314553, 7.315218, 7.315884, 7.316548, 7.317212, 7.317876, 7.318540, 7.319202,
7.319865, 7.320527, 7.321189, 7.321850, 7.322510, 7.323171, 7.323831, 7.324490, 7.325149, 7.325808,
7.326466, 7.327123, 7.327781, 7.328437, 7.329094, 7.329750, 7.330405, 7.331060, 7.331715, 7.332369,
7.333023, 7.333676, 7.334329, 7.334982, 7.335634, 7.336286, 7.336937, 7.337588, 7.338238, 7.338888,
7.339538, 7.340187, 7.340836, 7.341484, 7.342132, 7.342779, 7.343426, 7.344073, 7.344719, 7.345365,
7.346010, 7.346655, 7.347300, 7.347944, 7.348588, 7.349231, 7.349874, 7.350516, 7.351158, 7.351800,
7.352441, 7.353082, 7.353722, 7.354362, 7.355002, 7.355641, 7.356280, 7.356918, 7.357556, 7.358194,
7.358831, 7.359468, 7.360104, 7.360740, 7.361375, 7.362011, 7.362645, 7.363280, 7.363914, 7.364547,
7.365180, 7.365813, 7.366445, 7.367077, 7.367709, 7.368340, 7.368970, 7.369601, 7.370231, 7.370860,
7.371489, 7.372118, 7.372746, 7.373374, 7.374002, 7.374629, 7.375256, 7.375882, 7.376508, 7.377134,
7.377759, 7.378384, 7.379008, 7.379632, 7.380256, 7.380879, 7.381502, 7.382124, 7.382746, 7.383368,
7.383989, 7.384610, 7.385231, 7.385851, 7.386471, 7.387090, 7.387709, 7.388328, 7.388946, 7.389564,
7.390181, 7.390799, 7.391415, 7.392032, 7.392648, 7.393263, 7.393878, 7.394493, 7.395108, 7.395722,
7.396335, 7.396949, 7.397562, 7.398174, 7.398786, 7.399398, 7.400010, 7.400621, 7.401231, 7.401842,
7.402452, 7.403061, 7.403670, 7.404279, 7.404888, 7.405496, 7.406103, 7.406711, 7.407318, 7.407924,
7.408531, 7.409136, 7.409742, 7.410347, 7.410952, 7.411556, 7.412160, 7.412764, 7.413367, 7.413970,
7.414573, 7.415175, 7.415777, 7.416378, 7.416980, 7.417580, 7.418181, 7.418781, 7.419381, 7.419980,
7.420579, 7.421178, 7.421776, 7.422374, 7.422971, 7.423568, 7.424165, 7.424762, 7.425358, 7.425954,
7.426549, 7.427144, 7.427739, 7.428333, 7.428927, 7.429521, 7.430114, 7.430707, 7.431300, 7.431892,
7.432484, 7.433075, 7.433667, 7.434257, 7.434848, 7.435438, 7.436028, 7.436617, 7.437206, 7.437795,
7.438384, 7.438972, 7.439559, 7.440147, 7.440734, 7.441320, 7.441907, 7.442493, 7.443078, 7.443664,
7.444249, 7.444833, 7.445418, 7.446001, 7.446585, 7.447168, 7.447751, 7.448334, 7.448916, 7.449498,
7.450080, 7.450661, 7.451242, 7.451822, 7.452402, 7.452982, 7.453562, 7.454141, 7.454720, 7.455298,
7.455877, 7.456455, 7.457032, 7.457609, 7.458186, 7.458763, 7.459339, 7.459915, 7.460490, 7.461066,
7.461640, 7.462215, 7.462789, 7.463363, 7.463937, 7.464510, 7.465083, 7.465655, 7.466228, 7.466799,
7.467371, 7.467942, 7.468513, 7.469084, 7.469654, 7.470224, 7.470794, 7.471363, 7.471932, 7.472501,
7.473069, 7.473637, 7.474205, 7.474772, 7.475339, 7.475906, 7.476472, 7.477038, 7.477604, 7.478170,
7.478735, 7.479300, 7.479864, 7.480428, 7.480992, 7.481556, 7.482119, 7.482682, 7.483244, 7.483807,
7.484369, 7.484930, 7.485492, 7.486053, 7.486613, 7.487174, 7.487734, 7.488294, 7.488853, 7.489412,
7.489971, 7.490529, 7.491088, 7.491645, 7.492203, 7.492760, 7.493317, 7.493874, 7.494430, 7.494986,
7.495542, 7.496097, 7.496652, 7.497207, 7.497762, 7.498316, 7.498870, 7.499423, 7.499977, 7.500529,
7.501082, 7.501634, 7.502186, 7.502738, 7.503290, 7.503841, 7.504392, 7.504942, 7.505492, 7.506042,
7.506592, 7.507141, 7.507690, 7.508239, 7.508787, 7.509335, 7.509883, 7.510431, 7.510978, 7.511525,
7.512071, 7.512618, 7.513164, 7.513709, 7.514255, 7.514800, 7.515345, 7.515889, 7.516433, 7.516977,
7.517521, 7.518064, 7.518607, 7.519150, 7.519692, 7.520235, 7.520776, 7.521318, 7.521859, 7.522400,
7.522941, 7.523481, 7.524021, 7.524561, 7.525101, 7.525640, 7.526179, 7.526718, 7.527256, 7.527794,
7.528332, 7.528869, 7.529406, 7.529943, 7.530480, 7.531016, 7.531552, 7.532088, 7.532624, 7.533159,
7.533694, 7.534228, 7.534763, 7.535297, 7.535830, 7.536364, 7.536897, 7.537430, 7.537963, 7.538495,
7.539027, 7.539559, 7.540090, 7.540622, 7.541152, 7.541683, 7.542213, 7.542744, 7.543273, 7.543803,
7.544332, 7.544861, 7.545390, 7.545918, 7.546446, 7.546974, 7.547502, 7.548029, 7.548556, 7.549083,
7.549609, 7.550135, 7.550661, 7.551187, 7.551712, 7.552237, 7.552762, 7.553287, 7.553811, 7.554335,
7.554859, 7.555382, 7.555905, 7.556428, 7.556951, 7.557473, 7.557995, 7.558517, 7.559038, 7.559559,
7.560080, 7.560601, 7.561122, 7.561642, 7.562162, 7.562681, 7.563201, 7.563720, 7.564238, 7.564757,
7.565275, 7.565793, 7.566311, 7.566828, 7.567346, 7.567863, 7.568379, 7.568896, 7.569412, 7.569928,
7.570443, 7.570959, 7.571474, 7.571988, 7.572503, 7.573017, 7.573531, 7.574045, 7.574558, 7.575072,
7.575585, 7.576097, 7.576610, 7.577122, 7.577634, 7.578145, 7.578657, 7.579168, 7.579679, 7.580189,
7.580700, 7.581210, 7.581720, 7.582229, 7.582738, 7.583248, 7.583756, 7.584265, 7.584773, 7.585281,
7.585789, 7.586296, 7.586804, 7.587311, 7.587817, 7.588324, 7.588830, 7.589336, 7.589842, 7.590347,
7.590852, 7.591357, 7.591862, 7.592366, 7.592870, 7.593374, 7.593878, 7.594381, 7.594884, 7.595387,
7.595890, 7.596392, 7.596894, 7.597396, 7.597898, 7.598399, 7.598900, 7.599401, 7.599902, 7.600402,
7.600902, 7.601402, 7.601902, 7.602401, 7.602900, 7.603399, 7.603898, 7.604396, 7.604894, 7.605392,
7.605890, 7.606387, 7.606885, 7.607381, 7.607878, 7.608374, 7.608871, 7.609367, 7.609862, 7.610358,
7.610853, 7.611348, 7.611842, 7.612337, 7.612831, 7.613325, 7.613819, 7.614312, 7.614805, 7.615298,
7.615791, 7.616284, 7.616776, 7.617268, 7.617760, 7.618251, 7.618742, 7.619233, 7.619724, 7.620215,
7.620705, 7.621195, 7.621685, 7.622175, 7.622664, 7.623153, 7.623642, 7.624131, 7.624619, 7.625107,
7.625595, 7.626083, 7.626570, 7.627057, 7.627544, 7.628031, 7.628518, 7.629004, 7.629490, 7.629976,
7.630461, 7.630947, 7.631432, 7.631917, 7.632401, 7.632886, 7.633370, 7.633854, 7.634337, 7.634821,
7.635304, 7.635787, 7.636270, 7.636752, 7.637234, 7.637716, 7.638198, 7.638680, 7.639161, 7.639642,
7.640123, 7.640604, 7.641084, 7.641564, 7.642044, 7.642524, 7.643004, 7.643483, 7.643962, 7.644441,
7.644919, 7.645398, 7.645876, 7.646354, 7.646831, 7.647309, 7.647786, 7.648263, 7.648740, 7.649216,
7.649693, 7.650169, 7.650645, 7.651120, 7.651596, 7.652071, 7.652546, 7.653020, 7.653495, 7.653969,
7.654443, 7.654917, 7.655391, 7.655864, 7.656337, 7.656810, 7.657283, 7.657755, 7.658228, 7.658700,
7.659171, 7.659643, 7.660114, 7.660585, 7.661056, 7.661527, 7.661998, 7.662468, 7.662938, 7.663408,
7.663877, 7.664347, 7.664816, 7.665285, 7.665753, 7.666222, 7.666690, 7.667158, 7.667626, 7.668094,
7.668561, 7.669028, 7.669495, 7.669962, 7.670429, 7.670895, 7.671361, 7.671827, 7.672292, 7.672758,
7.673223, 7.673688, 7.674153, 7.674617, 7.675082, 7.675546, 7.676010, 7.676474, 7.676937, 7.677400,
7.677864, 7.678326, 7.678789, 7.679251, 7.679714, 7.680176, 7.680637, 7.681099, 7.681560, 7.682022,
7.682482, 7.682943, 7.683404, 7.683864, 7.684324, 7.684784, 7.685244, 7.685703, 7.686162, 7.686621,
7.687080, 7.687539, 7.687997, 7.688455, 7.688913, 7.689371, 7.689829, 7.690286, 7.690743, 7.691200,
7.691657, 7.692113, 7.692570, 7.693026, 7.693482, 7.693937, 7.694393, 7.694848, 7.695303, 7.695758,
7.696213, 7.696667, 7.697121, 7.697575, 7.698029, 7.698483, 7.698936, 7.699389, 7.699842, 7.700295,
7.700748, 7.701200, 7.701652, 7.702104, 7.702556, 7.703008, 7.703459, 7.703910, 7.704361, 7.704812,
7.705262, 7.705713, 7.706163, 7.706613, 7.707063, 7.707512, 7.707962, 7.708411, 7.708860, 7.709308,
7.709757, 7.710205, 7.710653, 7.711101, 7.711549, 7.711997, 7.712444, 7.712891, 7.713338, 7.713785,
7.714231, 7.714677, 7.715124, 7.715570, 7.716015, 7.716461, 7.716906, 7.717351, 7.717796, 7.718241,
7.718685, 7.719130, 7.719574, 7.720018, 7.720462, 7.720905, 7.721349, 7.721792, 7.722235, 7.722678,
7.723120, 7.723562, 7.724005, 7.724447, 7.724888, 7.725330, 7.725771, 7.726213, 7.726654, 7.727094,
7.727535, 7.727976, 7.728416, 7.728856, 7.729296, 7.729735, 7.730175, 7.730614, 7.731053, 7.731492,
7.731931, 7.732369, 7.732808, 7.733246, 7.733684, 7.734121, 7.734559, 7.734996, 7.735433, 7.735870,
7.736307, 7.736744, 7.737180, 7.737616, 7.738052, 7.738488, 7.738924, 7.739359, 7.739794, 7.740230,
7.740664, 7.741099, 7.741534, 7.741968, 7.742402, 7.742836, 7.743270, 7.743703, 7.744137, 7.744570,
7.745003, 7.745436, 7.745868, 7.746301, 7.746733, 7.747165, 7.747597, 7.748029, 7.748460, 7.748891,
7.749322, 7.749753, 7.750184, 7.750615, 7.751045, 7.751475, 7.751905, 7.752335, 7.752765, 7.753194,
7.753624, 7.754053, 7.754482, 7.754910, 7.755339, 7.755767, 7.756195, 7.756623, 7.757051, 7.757479,
7.757906, 7.758333, 7.758761, 7.759187, 7.759614, 7.760041, 7.760467, 7.760893, 7.761319, 7.761745,
7.762171, 7.762596, 7.763021, 7.763446, 7.763871, 7.764296, 7.764721, 7.765145, 7.765569, 7.765993,
7.766417, 7.766841, 7.767264, 7.767687, 7.768110, 7.768533, 7.768956, 7.769379, 7.769801, 7.770223,
7.770645, 7.771067, 7.771489, 7.771910, 7.772332, 7.772753, 7.773174, 7.773594, 7.774015, 7.774436,
7.774856, 7.775276, 7.775696, 7.776115, 7.776535, 7.776954, 7.777374, 7.777793, 7.778211, 7.778630,
7.779049, 7.779467, 7.779885, 7.780303, 7.780721, 7.781139, 7.781556, 7.781973, 7.782390, 7.782807,
7.783224, 7.783641, 7.784057, 7.784473, 7.784889, 7.785305, 7.785721, 7.786136, 7.786552, 7.786967,
7.787382, 7.787797, 7.788212, 7.788626, 7.789040, 7.789455, 7.789869, 7.790282, 7.790696, 7.791110,
7.791523, 7.791936, 7.792349, 7.792762, 7.793174, 7.793587, 7.793999, 7.794411, 7.794823, 7.795235,
7.795647, 7.796058, 7.796469, 7.796880, 7.797291, 7.797702, 7.798113, 7.798523, 7.798933, 7.799343,
7.799753, 7.800163, 7.800573, 7.800982, 7.801391, 7.801800, 7.802209, 7.802618, 7.803027, 7.803435,
7.803843, 7.804251, 7.804659, 7.805067, 7.805475, 7.805882, 7.806289, 7.806696, 7.807103, 7.807510,
7.807917, 7.808323, 7.808729, 7.809135, 7.809541, 7.809947, 7.810353, 7.810758, 7.811163, 7.811568,
7.811973, 7.812378, 7.812783, 7.813187, 7.813592, 7.813996, 7.814400, 7.814803, 7.815207, 7.815611,
7.816014, 7.816417, 7.816820, 7.817223, 7.817625, 7.818028, 7.818430, 7.818832, 7.819234, 7.819636,
7.820038, 7.820440, 7.820841, 7.821242, 7.821643, 7.822044, 7.822445, 7.822845, 7.823246, 7.823646,
7.824046, 7.824446, 7.824846, 7.825245, 7.825645, 7.826044, 7.826443, 7.826842, 7.827241, 7.827640,
7.828038, 7.828436, 7.828835, 7.829233, 7.829630, 7.830028, 7.830426, 7.830823, 7.831220, 7.831617,
7.832014, 7.832411, 7.832808, 7.833204, 7.833600, 7.833996, 7.834392, 7.834788, 7.835184, 7.835579,
7.835975, 7.836370, 7.836765, 7.837160, 7.837554, 7.837949, 7.838343, 7.838738, 7.839132, 7.839526,
7.839919, 7.840313, 7.840706, 7.841100, 7.841493, 7.841886, 7.842279, 7.842671, 7.843064, 7.843456,
7.843849, 7.844241, 7.844633, 7.845024, 7.845416, 7.845808, 7.846199, 7.846590, 7.846981, 7.847372,
7.847763, 7.848153, 7.848543, 7.848934, 7.849324, 7.849714, 7.850104, 7.850493, 7.850883, 7.851272,
7.851661, 7.852050, 7.852439, 7.852828, 7.853216, 7.853605, 7.853993, 7.854381, 7.854769, 7.855157,
7.855545, 7.855932, 7.856320, 7.856707, 7.857094, 7.857481, 7.857868, 7.858254, 7.858641, 7.859027,
7.859413, 7.859799, 7.860185, 7.860571, 7.860956, 7.861342, 7.861727, 7.862112, 7.862497, 7.862882,
7.863267, 7.863651, 7.864036, 7.864420, 7.864804, 7.865188, 7.865572, 7.865955, 7.866339, 7.866722,
7.867106, 7.867489, 7.867871, 7.868254, 7.868637, 7.869019, 7.869402, 7.869784, 7.870166, 7.870548,
7.870930, 7.871311, 7.871693, 7.872074, 7.872455, 7.872836, 7.873217, 7.873598, 7.873978, 7.874359,
7.874739, 7.875119, 7.875499, 7.875879, 7.876259, 7.876638, 7.877018, 7.877397, 7.877776, 7.878155,
7.878534, 7.878913, 7.879291, 7.879670, 7.880048, 7.880426, 7.880804, 7.881182, 7.881560, 7.881937,
7.882315, 7.882692, 7.883069, 7.883446, 7.883823, 7.884200, 7.884577, 7.884953, 7.885329, 7.885705,
7.886081, 7.886457, 7.886833, 7.887209, 7.887584, 7.887959, 7.888335, 7.888710, 7.889084, 7.889459,
7.889834, 7.890208, 7.890583, 7.890957, 7.891331, 7.891705, 7.892078, 7.892452, 7.892826, 7.893199,
7.893572, 7.893945, 7.894318, 7.894691, 7.895063, 7.895436, 7.895808, 7.896181, 7.896553, 7.896925,
7.897296, 7.897668, 7.898040, 7.898411, 7.898782, 7.899153, 7.899524, 7.899895, 7.900266, 7.900637,
7.901007, 7.901377, 7.901748, 7.902118, 7.902487, 7.902857, 7.903227, 7.903596, 7.903966, 7.904335,
7.904704, 7.905073, 7.905442, 7.905810, 7.906179, 7.906547, 7.906915, 7.907284, 7.907652, 7.908019,
7.908387, 7.908755, 7.909122, 7.909489, 7.909857, 7.910224, 7.910591, 7.910957, 7.911324, 7.911691,
7.912057, 7.912423, 7.912789, 7.913155, 7.913521, 7.913887, 7.914252, 7.914618, 7.914983, 7.915348,
7.915713, 7.916078, 7.916443, 7.916807, 7.917172, 7.917536, 7.917901, 7.918265, 7.918629, 7.918992,
7.919356, 7.919720, 7.920083, 7.920447, 7.920810, 7.921173, 7.921536, 7.921898, 7.922261, 7.922624,
7.922986, 7.923348, 7.923710, 7.924072, 7.924434, 7.924796, 7.925158, 7.925519, 7.925880, 7.926242,
7.926603, 7.926964, 7.927324, 7.927685, 7.928046, 7.928406, 7.928766, 7.929126, 7.929487, 7.929846,
7.930206, 7.930566, 7.930925, 7.931285, 7.931644, 7.932003, 7.932362, 7.932721, 7.933080, 7.933438,
7.933797, 7.934155, 7.934513, 7.934872, 7.935230, 7.935587, 7.935945, 7.936303, 7.936660, 7.937017,
7.937375, 7.937732, 7.938089, 7.938446, 7.938802, 7.939159, 7.939515, 7.939872, 7.940228, 7.940584,
7.940940, 7.941296, 7.941651, 7.942007, 7.942362, 7.942718, 7.943073, 7.943428, 7.943783, 7.944137,
7.944492, 7.944847, 7.945201, 7.945555, 7.945910, 7.946264, 7.946618, 7.946971, 7.947325, 7.947679,
7.948032, 7.948385, 7.948738, 7.949091, 7.949444, 7.949797, 7.950150, 7.950502, 7.950855, 7.951207,
7.951559, 7.951911, 7.952263, 7.952615, 7.952967, 7.953318, 7.953670, 7.954021, 7.954372, 7.954723,
7.955074, 7.955425, 7.955776, 7.956126, 7.956477, 7.956827, 7.957177, 7.957527, 7.957877, 7.958227,
7.958577, 7.958926, 7.959276, 7.959625, 7.959975, 7.960324, 7.960673, 7.961021, 7.961370, 7.961719,
7.962067, 7.962416, 7.962764, 7.963112, 7.963460, 7.963808, 7.964156, 7.964503, 7.964851, 7.965198,
7.965546, 7.965893, 7.966240, 7.966587, 7.966933, 7.967280, 7.967627, 7.967973, 7.968320, 7.968666,
7.969012, 7.969358, 7.969704, 7.970049, 7.970395, 7.970740, 7.971086, 7.971431, 7.971776, 7.972121,
7.972466, 7.972811, 7.973155, 7.973500, 7.973844, 7.974189, 7.974533, 7.974877, 7.975221, 7.975565,
7.975908, 7.976252, 7.976595, 7.976939, 7.977282, 7.977625, 7.977968, 7.978311, 7.978654, 7.978996,
7.979339, 7.979681, 7.980024, 7.980366, 7.980708, 7.981050, 7.981392, 7.981733, 7.982075, 7.982416,
7.982758, 7.983099, 7.983440, 7.983781, 7.984122, 7.984463, 7.984803, 7.985144, 7.985484, 7.985825,
7.986165, 7.986505, 7.986845, 7.987185, 7.987524, 7.987864, 7.988204, 7.988543, 7.988882, 7.989221,
7.989560, 7.989899, 7.990238, 7.990577, 7.990915, 7.991254, 7.991592, 7.991931, 7.992269, 7.992607,
7.992945, 7.993282, 7.993620, 7.993958, 7.994295, 7.994632, 7.994970, 7.995307, 7.995644, 7.995980,
7.996317, 7.996654, 7.996990, 7.997327, 7.997663, 7.997999, 7.998335, 7.998671, 7.999007, 7.999343,
7.999679, 8.000014, 8.000349, 8.000685, 8.001020, 8.001355, 8.001690, 8.002025, 8.002360, 8.002694,
8.003029, 8.003363, 8.003697, 8.004032, 8.004366, 8.004700, 8.005033, 8.005367, 8.005701, 8.006034,
8.006368, 8.006701, 8.007034, 8.007367, 8.007700, 8.008033, 8.008366, 8.008698, 8.009031, 8.009363,
8.009695, 8.010028, 8.010360, 8.010692, 8.011023, 8.011355, 8.011687, 8.012018, 8.012350, 8.012681,
8.013012, 8.013343, 8.013674, 8.014005, 8.014336, 8.014666, 8.014997, 8.015327, 8.015658, 8.015988,
8.016318, 8.016648, 8.016978, 8.017308, 8.017637, 8.017967, 8.018296, 8.018625, 8.018955, 8.019284,
8.019613, 8.019942, 8.020270, 8.020599, 8.020928, 8.021256, 8.021585, 8.021913, 8.022241, 8.022569,
8.022897, 8.023225, 8.023552, 8.023880, 8.024207, 8.024535, 8.024862, 8.025189, 8.025516, 8.025843,
8.026170, 8.026497, 8.026824, 8.027150, 8.027477, 8.027803, 8.028129, 8.028455, 8.028781, 8.029107,
8.029433, 8.029759, 8.030084, 8.030410, 8.030735, 8.031060, 8.031385, 8.031710, 8.032035, 8.032360,
8.032685, 8.033009, 8.033334, 8.033658, 8.033983, 8.034307, 8.034631, 8.034955, 8.035279, 8.035603,
8.035926, 8.036250, 8.036573, 8.036897, 8.037220, 8.037543, 8.037866, 8.038189, 8.038512, 8.038835,
8.039157, 8.039480, 8.039802, 8.040125, 8.040447, 8.040769, 8.041091, 8.041413, 8.041735, 8.042056,
8.042378, 8.042699, 8.043021, 8.043342, 8.043663, 8.043984, 8.044305, 8.044626, 8.044947, 8.045268,
8.045588, 8.045909, 8.046229, 8.046549, 8.046870, 8.047190, 8.047510, 8.047829, 8.048149, 8.048469,
8.048788, 8.049108, 8.049427, 8.049746, 8.050065, 8.050384, 8.050703, 8.051022, 8.051341, 8.051660,
8.051978, 8.052296, 8.052615, 8.052933, 8.053251, 8.053569, 8.053887, 8.054205, 8.054523, 8.054840,
8.055158, 8.055475, 8.055792, 8.056110, 8.056427, 8.056744, 8.057061, 8.057377, 8.057694, 8.058011,
8.058327, 8.058644, 8.058960, 8.059276, 8.059592, 8.059908, 8.060224, 8.060540, 8.060856, 8.061171,
8.061487, 8.061802, 8.062118, 8.062433, 8.062748, 8.063063, 8.063378, 8.063693, 8.064007, 8.064322,
8.064636, 8.064951, 8.065265, 8.065579, 8.065894, 8.066208, 8.066521, 8.066835, 8.067149, 8.067463,
8.067776, 8.068090, 8.068403, 8.068716, 8.069029, 8.069342, 8.069655, 8.069968, 8.070281, 8.070594,
8.070906, 8.071219, 8.071531, 8.071843, 8.072155, 8.072467, 8.072779, 8.073091, 8.073403, 8.073715,
8.074026, 8.074338, 8.074649, 8.074960, 8.075272, 8.075583, 8.075894, 8.076205, 8.076515, 8.076826,
8.077137, 8.077447, 8.077758, 8.078068, 8.078378, 8.078688, 8.078998, 8.079308, 8.079618, 8.079928,
8.080237, 8.080547, 8.080856, 8.081166, 8.081475, 8.081784, 8.082093, 8.082402, 8.082711, 8.083020,
8.083329, 8.083637, 8.083946, 8.084254, 8.084562, 8.084871, 8.085179, 8.085487, 8.085795, 8.086103,
8.086410, 8.086718, 8.087025, 8.087333, 8.087640, 8.087948, 8.088255, 8.088562, 8.088869, 8.089176,
8.089482, 8.089789, 8.090096, 8.090402, 8.090709, 8.091015, 8.091321, 8.091627, 8.091933, 8.092239,
8.092545, 8.092851, 8.093157, 8.093462, 8.093768, 8.094073, 8.094378, 8.094684, 8.094989, 8.095294,
8.095599, 8.095904, 8.096208, 8.096513, 8.096817, 8.097122, 8.097426, 8.097731, 8.098035, 8.098339,
8.098643, 8.098947, 8.099251, 8.099554, 8.099858, 8.100161, 8.100465, 8.100768, 8.101072, 8.101375,
8.101678, 8.101981, 8.102284, 8.102586, 8.102889, 8.103192, 8.103494, 8.103797, 8.104099, 8.104401,
8.104703, 8.105006, 8.105308, 8.105609, 8.105911, 8.106213, 8.106515, 8.106816, 8.107117, 8.107419,
8.107720, 8.108021, 8.108322, 8.108623, 8.108924, 8.109225, 8.109526, 8.109826, 8.110127, 8.110427,
8.110728, 8.111028, 8.111328, 8.111628, 8.111928, 8.112228, 8.112528, 8.112827, 8.113127, 8.113427,
8.113726, 8.114025, 8.114325, 8.114624, 8.114923, 8.115222, 8.115521, 8.115820, 8.116118, 8.116417,
8.116716, 8.117014, 8.117312, 8.117611, 8.117909, 8.118207, 8.118505, 8.118803, 8.119101, 8.119399,
8.119696, 8.119994, 8.120291, 8.120589, 8.120886, 8.121183, 8.121480, 8.121777, 8.122074, 8.122371,
8.122668, 8.122965, 8.123261, 8.123558, 8.123854, 8.124151, 8.124447, 8.124743, 8.125039, 8.125335,
8.125631, 8.125927, 8.126223, 8.126518, 8.126814, 8.127109, 8.127405, 8.127700, 8.127995, 8.128290,
8.128585, 8.128880, 8.129175, 8.129470, 8.129764, 8.130059, 8.130354, 8.130648, 8.130942, 8.131237,
8.131531, 8.131825, 8.132119, 8.132413, 8.132706, 8.133000, 8.133294, 8.133587, 8.133881, 8.134174,
8.134468, 8.134761, 8.135054, 8.135347, 8.135640, 8.135933, 8.136226, 8.136518, 8.136811, 8.137103,
8.137396, 8.137688, 8.137980, 8.138273, 8.138565, 8.138857, 8.139149, 8.139441, 8.139732, 8.140024,
8.140316, 8.140607, 8.140898, 8.141190, 8.141481, 8.141772, 8.142063, 8.142354, 8.142645, 8.142936,
8.143227, 8.143517, 8.143808, 8.144098, 8.144389, 8.144679, 8.144969, 8.145260, 8.145550, 8.145840,
8.146130, 8.146419, 8.146709, 8.146999, 8.147288, 8.147578, 8.147867, 8.148156, 8.148446, 8.148735,
8.149024, 8.149313, 8.149602, 8.149891, 8.150179, 8.150468, 8.150756, 8.151045, 8.151333, 8.151622,
8.151910, 8.152198, 8.152486, 8.152774, 8.153062, 8.153350, 8.153637, 8.153925, 8.154213, 8.154500,
8.154788, 8.155075, 8.155362, 8.155649, 8.155936, 8.156223, 8.156510, 8.156797, 8.157084, 8.157370,
8.157657, 8.157944, 8.158230, 8.158516, 8.158802, 8.159089, 8.159375, 8.159661, 8.159947, 8.160232,
8.160518, 8.160804, 8.161090, 8.161375, 8.161660, 8.161946, 8.162231, 8.162516, 8.162801, 8.163086,
8.163371, 8.163656, 8.163941, 8.164226, 8.164510, 8.164795, 8.165079, 8.165364, 8.165648, 8.165932,
8.166216, 8.166500, 8.166784, 8.167068, 8.167352, 8.167636, 8.167919, 8.168203, 8.168486, 8.168770,
8.169053, 8.169336, 8.169620, 8.169903, 8.170186, 8.170469, 8.170751, 8.171034, 8.171317, 8.171599,
8.171882, 8.172164, 8.172447, 8.172729, 8.173011, 8.173293, 8.173575, 8.173857, 8.174139, 8.174421,
8.174703, 8.174985, 8.175266, 8.175548, 8.175829, 8.176110, 8.176392, 8.176673, 8.176954, 8.177235,
8.177516, 8.177797, 8.178077, 8.178358, 8.178639, 8.178919, 8.179200, 8.179480, 8.179760, 8.180041,
8.180321, 8.180601, 8.180881, 8.181161, 8.181441, 8.181720, 8.182000, 8.182280, 8.182559, 8.182839,
8.183118, 8.183397, 8.183677, 8.183956, 8.184235, 8.184514, 8.184793, 8.185071, 8.185350, 8.185629,
8.185907, 8.186186, 8.186464, 8.186743, 8.187021, 8.187299, 8.187577, 8.187855, 8.188133, 8.188411,
8.188689, 8.188967, 8.189245, 8.189522, 8.189800, 8.190077, 8.190354, 8.190632, 8.190909, 8.191186,
8.191463, 8.191740, 8.192017, 8.192294, 8.192570, 8.192847, 8.193124, 8.193400, 8.193677, 8.193953,
8.194229, 8.194506, 8.194782, 8.195058, 8.195334, 8.195610, 8.195885, 8.196161, 8.196437, 8.196712,
8.196988, 8.197263, 8.197539, 8.197814, 8.198089, 8.198364, 8.198639, 8.198914, 8.199189, 8.199464,
8.199739, 8.200014, 8.200288, 8.200563, 8.200837, 8.201112, 8.201386, 8.201660, 8.201934, 8.202208,
8.202482, 8.202756, 8.203030, 8.203304, 8.203578, 8.203851, 8.204125, 8.204398, 8.204672, 8.204945,
8.205218, 8.205492, 8.205765, 8.206038, 8.206311, 8.206584, 8.206856, 8.207129, 8.207402, 8.207674,
8.207947, 8.208219, 8.208492, 8.208764, 8.209036, 8.209308, 8.209580, 8.209852, 8.210124, 8.210396,
8.210668, 8.210940, 8.211211, 8.211483, 8.211754, 8.212026, 8.212297, 8.212568, 8.212840, 8.213111,
8.213382, 8.213653, 8.213924, 8.214194, 8.214465, 8.214736, 8.215006, 8.215277, 8.215547, 8.215818,
8.216088, 8.216358, 8.216628, 8.216899, 8.217169, 8.217439, 8.217708, 8.217978, 8.218248, 8.218518,
8.218787, 8.219057, 8.219326, 8.219595, 8.219865, 8.220134, 8.220403, 8.220672, 8.220941, 8.221210,
8.221479, 8.221748, 8.222016, 8.222285, 8.222554, 8.222822, 8.223091, 8.223359, 8.223627, 8.223895,
8.224164, 8.224432, 8.224700, 8.224967, 8.225235, 8.225503, 8.225771, 8.226038, 8.226306, 8.226573,
8.226841, 8.227108, 8.227376, 8.227643, 8.227910, 8.228177, 8.228444, 8.228711, 8.228978, 8.229244,
8.229511, 8.229778, 8.230044, 8.230311, 8.230577, 8.230844, 8.231110, 8.231376, 8.231642, 8.231908,
8.232174, 8.232440, 8.232706, 8.232972, 8.233238, 8.233503, 8.233769, 8.234034, 8.234300, 8.234565,
8.234830, 8.235095, 8.235361, 8.235626, 8.235891, 8.236156, 8.236421, 8.236685, 8.236950, 8.237215,
8.237479, 8.237744, 8.238008, 8.238273, 8.238537, 8.238801, 8.239065, 8.239329, 8.239593, 8.239857,
8.240121, 8.240385, 8.240649, 8.240913, 8.241176, 8.241440, 8.241703, 8.241967, 8.242230, 8.242493,
8.242756, 8.243019, 8.243283, 8.243546, 8.243808, 8.244071, 8.244334, 8.244597, 8.244859, 8.245122,
8.245384, 8.245647, 8.245909, 8.246172, 8.246434, 8.246696, 8.246958, 8.247220, 8.247482, 8.247744,
8.248006, 8.248267, 8.248529, 8.248791, 8.249052, 8.249314, 8.249575, 8.249836, 8.250098, 8.250359,
8.250620, 8.250881, 8.251142, 8.251403, 8.251664, 8.251925, 8.252185, 8.252446, 8.252707, 8.252967,
8.253228, 8.253488, 8.253748, 8.254009, 8.254269, 8.254529, 8.254789, 8.255049, 8.255309, 8.255569,
8.255828, 8.256088, 8.256348, 8.256607, 8.256867, 8.257126, 8.257386, 8.257645, 8.257904, 8.258163,
8.258422, 8.258681, 8.258940, 8.259199, 8.259458, 8.259717, 8.259976, 8.260234, 8.260493, 8.260751,
8.261010, 8.261268, 8.261526, 8.261785, 8.262043, 8.262301, 8.262559, 8.262817, 8.263075, 8.263333,
8.263590, 8.263848, 8.264106, 8.264363, 8.264621, 8.264878, 8.265136, 8.265393, 8.265650, 8.265907,
8.266164, 8.266421, 8.266678, 8.266935, 8.267192, 8.267449, 8.267706, 8.267962, 8.268219, 8.268475,
8.268732, 8.268988, 8.269245, 8.269501, 8.269757, 8.270013, 8.270269, 8.270525, 8.270781, 8.271037,
8.271293, 8.271548, 8.271804, 8.272060, 8.272315, 8.272571, 8.272826, 8.273081, 8.273337, 8.273592,
8.273847, 8.274102, 8.274357, 8.274612, 8.274867, 8.275122, 8.275376, 8.275631, 8.275886, 8.276140,
8.276395, 8.276649, 8.276903, 8.277158, 8.277412, 8.277666, 8.277920, 8.278174, 8.278428, 8.278682,
8.278936, 8.279190, 8.279443, 8.279697, 8.279951, 8.280204, 8.280458, 8.280711, 8.280964, 8.281218,
8.281471, 8.281724, 8.281977, 8.282230, 8.282483, 8.282736, 8.282989, 8.283241, 8.283494, 8.283747,
8.283999, 8.284252, 8.284504, 8.284757, 8.285009, 8.285261, 8.285513, 8.285765, 8.286017, 8.286269,
8.286521, 8.286773, 8.287025, 8.287277, 8.287528, 8.287780, 8.288032, 8.288283, 8.288534, 8.288786,
8.289037, 8.289288, 8.289539, 8.289791, 8.290042, 8.290293, 8.290544, 8.290794, 8.291045, 8.291296,
8.291547, 8.291797, 8.292048, 8.292298, 8.292549, 8.292799, 8.293049, 8.293299, 8.293550, 8.293800,
      };

   if (x<0.99)
   {
      mexPrintf("Integer log function not accurate\n");
      return log(x);
   }
   else if (x>=4000)
      return log(x);
   else
      return temp_log[(int)(x+0.5)];
}




// calculates log( gamma( x/2 ) )
inline double my_sf_lngamma(int x)
{
   static const double temp[4000] = {
       0, 0.5723649429, 0.0000000000, -0.1207822376, -0.0000000000, 0.2846828705, 0.6931471806, 1.2009736023, 1.7917594692, 2.4537365708,
3.1780538303, 3.9578139676, 4.7874917428, 5.6625620599, 6.5792512120, 7.5343642368, 8.5251613611, 9.5492672573, 10.6046029027, 11.6893334208,
12.8018274801, 13.9406252194, 15.1044125731, 16.2920004766, 17.5023078459, 18.7343475119, 19.9872144957, 21.2600761562, 22.5521638531, 23.8627658417,
25.1912211827, 26.5369144911, 27.8992713838, 29.2777545150, 30.6718601061, 32.0811148960, 33.5050734501, 34.9433157769, 36.3954452080, 37.8610865090,
39.3398841872, 40.8315009745, 42.3356164608, 43.8519258607, 45.3801388985, 46.9199787958, 48.4711813518, 50.0334941050, 51.6066755678, 53.1904945262,
54.7847293981, 56.3891676437, 58.0036052230, 59.6278460959, 61.2617017610, 62.9049908289, 64.5575386270, 66.2191768336, 67.8897431372, 69.5690809208,
71.2570389672, 72.9534711842, 74.6582363489, 76.3711978678, 78.0922235533, 79.8211854136, 81.5579594561, 83.3024255030, 85.0544670176, 86.8139709418,
88.5808275422, 90.3549302659, 92.1361756037, 93.9244629623, 95.7196945422, 97.5217752229, 99.3306124548, 101.1461161559, 102.9681986146, 104.7967743972,
106.6317602607, 108.4730750691, 110.3206397148, 112.1743770432, 114.0342117815, 115.9000704705, 117.7718813998, 119.6495745464, 121.5330815155, 123.4223354845,
125.3172711494, 127.2178246737, 129.1239336392, 131.0355369996, 132.9525750357, 134.8749893122, 136.8027226374, 138.7357190233, 140.6739236483, 142.6172828212,
144.5657439464, 146.5192554908, 148.4777669518, 150.4412288271, 152.4095925846, 154.3828106347, 156.3608363031, 158.3436238043, 160.3311282167, 162.3233054582,
164.3201122633, 166.3215061599, 168.3274454485, 170.3378891807, 172.3527971392, 174.3721298188, 176.3958484071, 178.4239147666, 180.4562914176, 182.4929415209,
184.5338288615, 186.5789178334, 188.6281734237, 190.6815611984, 192.7390472879, 194.8005983733, 196.8661816730, 198.9357649300, 201.0093163994, 203.0868048359,
205.1681994827, 207.2534700597, 209.3425867526, 211.4355202023, 213.5322414946, 215.6327221500, 217.7369341140, 219.8448497479, 221.9564418192, 224.0716834932,
226.1905483238, 228.3130102457, 230.4390435659, 232.5686229556, 234.7017234429, 236.8383204053, 238.9783895619, 241.1219069671, 243.2688490031, 245.4191923733,
247.5729140963, 249.7299914987, 251.8904022098, 254.0541241550, 256.2211355501, 258.3914148958, 260.5649409720, 262.7416928322, 264.9216497986, 267.1047914570,
269.2910976511, 271.4805484786, 273.6731242858, 275.8688056630, 278.0675734405, 280.2694086833, 282.4742926877, 284.6822069766, 286.8931332955, 289.1070536085,
291.3239500944, 293.5438051429, 295.7666013509, 297.9923215188, 300.2209486471, 302.4524659327, 304.6868567658, 306.9241047261, 309.1641935802, 311.4071072781,
313.6528299500, 315.9013459034, 318.1526396203, 320.4066957541, 322.6634991268, 324.9230347264, 327.1852877039, 329.4502433709, 331.7178871970, 333.9882048072,
336.2611819793, 338.5368046417, 340.8150588709, 343.0959308892, 345.3794070624, 347.6654738975, 349.9541180409, 352.2453262755, 354.5390855195, 356.8353828237,
359.1342053697, 361.4355404679, 363.7393755557, 366.0456981954, 368.3544960725, 370.6657569939, 372.9794688858, 375.2956197924, 377.6141978740, 379.9351914051,
382.2585887732, 384.5843784766, 386.9125491233, 389.2430894295, 391.5759882174, 393.9112344146, 396.2488170519, 398.5887252622, 400.9309482790, 403.2754754352,
405.6222961613, 407.9713999844, 410.3227765270, 412.6764155054, 415.0323067284, 417.3904400963, 419.7508055997, 422.1133933179, 424.4781934184, 426.8451961549,
429.2143918668, 431.5857709778, 433.9593239951, 436.3350415078, 438.7129141862, 441.0929327808, 443.4750881210, 445.8593711144, 448.2457727455, 450.6342840750,
453.0248962386, 455.4176004463, 457.8123879814, 460.2092501993, 462.6081785270, 465.0091644621, 467.4121995717, 469.8172754920, 472.2243839271, 474.6335166481,
477.0446654927, 479.4578223640, 481.8729792300, 484.2901281226, 486.7092611370, 489.1303704308, 491.5534482234, 493.9784867954, 496.4054784873, 498.8344156997,
501.2652908917, 503.6980965808, 506.1328253422, 508.5694698076, 511.0080226654, 513.4484766592, 515.8908245879, 518.3350593046, 520.7811737162, 523.2291607825,
525.6790135161, 528.1307249815, 530.5842882946, 533.0396966218, 535.4969431803, 537.9560212365, 540.4169241061, 542.8796451536, 545.3441777913, 547.8105154792,
550.2786517244, 552.7485800805, 555.2202941470, 557.6937875692, 560.1690540374, 562.6460872863, 565.1248810950, 567.6054292860, 570.0877257253, 572.5717643212,
575.0575390248, 577.5450438288, 580.0342727673, 582.5252199154, 585.0178793890, 587.5122453439, 590.0083119757, 592.5060735196, 595.0055242495, 597.5066584779,
600.0094705555, 602.5139548707, 605.0201058496, 607.5279179549, 610.0373856864, 612.5485035799, 615.0612662072, 617.5756681759, 620.0917041286, 622.6093687429,
625.1286567310, 627.6495628393, 630.1720818479, 632.6962085709, 635.2219378552, 637.7492645809, 640.2781836605, 642.8086900391, 645.3407786936, 647.8744446324,
650.4096828958, 652.9464885547, 655.4848567110, 658.0247824972, 660.5662610760, 663.1092876399, 665.6538574112, 668.1999656417, 670.7476076120, 673.2967786320,
675.8474740399, 678.3996892022, 680.9534195138, 683.5086603970, 686.0654073021, 688.6236557065, 691.1834011145, 693.7446390577, 696.3073650939, 698.8715748075,
701.4372638089, 704.0044277343, 706.5730622459, 709.1431630311, 711.7147258024, 714.2877462977, 716.8622202792, 719.4381435341, 722.0155118737, 724.5943211335,
727.1745671730, 729.7562458752, 732.3393531469, 734.9238849181, 737.5098371419, 740.0972057944, 742.6859868745, 745.2761764034, 747.8677704248, 750.4607650046,
753.0551562306, 755.6509402125, 758.2481130815, 760.8466709903, 763.4466101128, 766.0479266440, 768.6506167999, 771.2546768170, 773.8601029527, 776.4668914845,
779.0750387103, 781.6845409480, 784.2953945354, 786.9075958300, 789.5211412091, 792.1360270691, 794.7522498260, 797.3698059145, 799.9886917888, 802.6089039214,
805.2304388038, 807.8532929459, 810.4774628760, 813.1029451405, 815.7297363040, 818.3578329491, 820.9872316761, 823.6179291029, 826.2499218650, 828.8832066153,
831.5177800240, 834.1536387784, 836.7907795826, 839.4291991578, 842.0688942418, 844.7098615891, 847.3520979706, 849.9956001735, 852.6403650013, 855.2863892736,
857.9336698260, 860.5822035099, 863.2319871925, 865.8830177567, 868.5352921006, 871.1888071381, 873.8435597980, 876.4995470246, 879.1567657770, 881.8152130295,
884.4748857709, 887.1357810050, 889.7978957500, 892.4612270389, 895.1257719188, 897.7915274513, 900.4584907121, 903.1266587910, 905.7960287918, 908.4665978322,
911.1383630438, 913.8113215716, 916.4854705745, 919.1608072247, 921.8373287079, 924.5150322232, 927.1939149826, 929.8739742116, 932.5552071483, 935.2376110442,
937.9211831634, 940.6059207826, 943.2918211915, 945.9788816921, 948.6670995992, 951.3564722396, 954.0469969527, 956.7386710901, 959.4314920155, 962.1254571046,
964.8205637453, 967.5168093372, 970.2141912917, 972.9127070320, 975.6123539932, 978.3131296215, 981.0150313751, 983.7180567231, 986.4222031465, 989.1274681372,
991.8338491984, 994.5413438444, 997.2499496006, 999.9596640033, 1002.6704845998, 1005.3824089482, 1008.0954346173, 1010.8095591866, 1013.5247802463, 1016.2410953970,
1018.9585022498, 1021.6769984265, 1024.3965815588, 1027.1172492889, 1029.8389992693, 1032.5618291624, 1035.2857366409, 1038.0107193875, 1040.7367750945, 1043.4639014646,
1046.1920962099, 1048.9213570525, 1051.6516817240, 1054.3830679659, 1057.1155135290, 1059.8490161738, 1062.5835736702, 1065.3191837973, 1068.0558443438, 1070.7935531076,
1073.5323078958, 1076.2721065245, 1079.0129468191, 1081.7548266140, 1084.4977437526, 1087.2416960871, 1089.9866814788, 1092.7326977975, 1095.4797429221, 1098.2278147400,
1100.9769111474, 1103.7270300489, 1106.4781693579, 1109.2303269962, 1111.9835008939, 1114.7376889897, 1117.4928892305, 1120.2490995717, 1123.0063179767, 1125.7645424172,
1128.5237708731, 1131.2840013324, 1134.0452317910, 1136.8074602529, 1139.5706847301, 1142.3349032425, 1145.1001138176, 1147.8663144912, 1150.6335033064, 1153.4016783142,
1156.1708375734, 1158.9409791502, 1161.7121011186, 1164.4842015599, 1167.2572785630, 1170.0313302245, 1172.8063546479, 1175.5823499446, 1178.3593142328, 1181.1372456385,
1183.9161422945, 1186.6960023411, 1189.4768239256, 1192.2586052023, 1195.0413443329, 1197.8250394858, 1200.6096888366, 1203.3952905679, 1206.1818428688, 1208.9693439358,
1211.7577919720, 1214.5471851871, 1217.3375217980, 1220.1288000278, 1222.9210181067, 1225.7141742714, 1228.5082667651, 1231.3032938378, 1234.0992537457, 1236.8961447517,
1239.6939651253, 1242.4927131421, 1245.2923870843, 1248.0929852403, 1250.8945059051, 1253.6969473797, 1256.5003079714, 1259.3045859938, 1262.1097797666, 1264.9158876157,
1267.7229078730, 1270.5308388766, 1273.3396789707, 1276.1494265052, 1278.9600798364, 1281.7716373262, 1284.5840973426, 1287.3974582594, 1290.2117184563, 1293.0268763187,
1295.8429302381, 1298.6598786113, 1301.4777198413, 1304.2964523363, 1307.1160745106, 1309.9365847839, 1312.7579815815, 1315.5802633345, 1318.4034284792, 1321.2274754576,
1324.0524027173, 1326.8782087112, 1329.7048918976, 1332.5324507403, 1335.3608837084, 1338.1901892764, 1341.0203659242, 1343.8514121367, 1346.6833264043, 1349.5161072226,
1352.3497530924, 1355.1842625196, 1358.0196340154, 1360.8558660959, 1363.6929572826, 1366.5309061017, 1369.3697110848, 1372.2093707684, 1375.0498836939, 1377.8912484077,
1380.7334634612, 1383.5765274108, 1386.4204388175, 1389.2651962476, 1392.1107982719, 1394.9572434660, 1397.8045304107, 1400.6526576910, 1403.5016238972, 1406.3514276239,
1409.2020674706, 1412.0535420414, 1414.9058499452, 1417.7589897954, 1420.6129602100, 1423.4677598116, 1426.3233872273, 1429.1798410890, 1432.0371200329, 1434.8952226995,
1437.7541477343, 1440.6138937867, 1443.4744595109, 1446.3358435653, 1449.1980446128, 1452.0610613207, 1454.9248923604, 1457.7895364079, 1460.6549921434, 1463.5212582513,
1466.3883334203, 1469.2562163434, 1472.1249057178, 1474.9944002448, 1477.8646986299, 1480.7357995830, 1483.6077018178, 1486.4804040522, 1489.3539050083, 1492.2282034122,
1495.1032979942, 1497.9791874884, 1500.8558706330, 1503.7333461704, 1506.6116128466, 1509.4906694119, 1512.3705146205, 1515.2511472303, 1518.1325660033, 1521.0147697053,
1523.8977571061, 1526.7815269791, 1529.6660781018, 1532.5514092555, 1535.4375192250, 1538.3244067991, 1541.2120707705, 1544.1005099354, 1546.9897230937, 1549.8797090493,
1552.7704666095, 1555.6619945855, 1558.5542917919, 1561.4473570471, 1564.3411891732, 1567.2357869958, 1570.1311493441, 1573.0272750509, 1575.9241629525, 1578.8218118888,
1581.7202207033, 1584.6193882428, 1587.5193133577, 1590.4199949021, 1593.3214317331, 1596.2236227117, 1599.1265667020, 1602.0302625718, 1604.9347091920, 1607.8399054371,
1610.7458501850, 1613.6525423167, 1616.5599807168, 1619.4681642731, 1622.3770918768, 1625.2867624222, 1628.1971748071, 1631.1083279325, 1634.0202207026, 1636.9328520249,
1639.8462208100, 1642.7603259719, 1645.6751664276, 1648.5907410975, 1651.5070489049, 1654.4240887765, 1657.3418596420, 1660.2603604342, 1663.1795900891, 1666.0995475458,
1669.0202317465, 1671.9416416363, 1674.8637761635, 1677.7866342795, 1680.7102149386, 1683.6345170981, 1686.5595397185, 1689.4852817631, 1692.4117421983, 1695.3389199933,
1698.2668141205, 1701.1954235550, 1704.1247472750, 1707.0547842615, 1709.9855334985, 1712.9169939727, 1715.8491646741, 1718.7820445950, 1721.7156327310, 1724.6499280803,
1727.5849296441, 1730.5206364263, 1733.4570474336, 1736.3941616755, 1739.3319781644, 1742.2704959154, 1745.2097139462, 1748.1496312775, 1751.0902469326, 1754.0315599376,
1756.9735693211, 1759.9162741147, 1762.8596733526, 1765.8037660715, 1768.7485513109, 1771.6940281131, 1774.6401955227, 1777.5870525874, 1780.5345983570, 1783.4828318843,
1786.4317522246, 1789.3813584358, 1792.3316495782, 1795.2826247150, 1798.2342829116, 1801.1866232363, 1804.1396447597, 1807.0933465549, 1810.0477276978, 1813.0027872666,
1815.9585243419, 1818.9149380070, 1821.8720273475, 1824.8297914517, 1827.7882294101, 1830.7473403159, 1833.7071232644, 1836.6675773537, 1839.6287016840, 1842.5904953582,
1845.5529574815, 1848.5160871612, 1851.4798835074, 1854.4443456324, 1857.4094726508, 1860.3752636796, 1863.3417178383, 1866.3088342483, 1869.2766120339, 1872.2450503212,
1875.2141482390, 1878.1839049181, 1881.1543194917, 1884.1253910954, 1887.0971188668, 1890.0695019461, 1893.0425394754, 1896.0162305993, 1898.9905744646, 1901.9655702202,
1904.9412170172, 1907.9175140091, 1910.8944603515, 1913.8720552021, 1916.8502977209, 1919.8291870701, 1922.8087224140, 1925.7889029189, 1928.7697277536, 1931.7511960887,
1934.7333070972, 1937.7160599540, 1940.6994538363, 1943.6834879233, 1946.6681613963, 1949.6534734388, 1952.6394232361, 1955.6260099760, 1958.6132328480, 1961.6010910438,
1964.5895837573, 1967.5787101841, 1970.5684695222, 1973.5588609713, 1976.5498837334, 1979.5415370124, 1982.5338200141, 1985.5267319465, 1988.5202720194, 1991.5144394447,
1994.5092334363, 1997.5046532099, 2000.5006979834, 2003.4973669764, 2006.4946594107, 2009.4925745098, 2012.4911114993, 2015.4902696067, 2018.4900480613, 2021.4904460944,
2024.4914629392, 2027.4930978309, 2030.4953500063, 2033.4982187045, 2036.5017031659, 2039.5058026334, 2042.5105163514, 2045.5158435661, 2048.5217835258, 2051.5283354804,
2054.5354986818, 2057.5432723837, 2060.5516558415, 2063.5606483126, 2066.5702490560, 2069.5804573327, 2072.5912724054, 2075.6026935385, 2078.6147199983, 2081.6273510530,
2084.6405859722, 2087.6544240275, 2090.6688644924, 2093.6839066418, 2096.6995497527, 2099.7157931034, 2102.7326359745, 2105.7500776477, 2108.7681174070, 2111.7867545377,
2114.8059883269, 2117.8258180636, 2120.8462430382, 2123.8672625430, 2126.8888758719, 2129.9110823204, 2132.9338811859, 2135.9572717672, 2138.9812533649, 2142.0058252813,
2145.0309868202, 2148.0567372871, 2151.0830759891, 2154.1100022351, 2157.1375153354, 2160.1656146020, 2163.1942993486, 2166.2235688904, 2169.2534225442, 2172.2838596284,
2175.3148794631, 2178.3464813698, 2181.3786646718, 2184.4114286937, 2187.4447727619, 2190.4786962042, 2193.5131983501, 2196.5482785306, 2199.5839360782, 2202.6201703268,
2205.6569806123, 2208.6943662715, 2211.7323266433, 2214.7708610678, 2217.8099688867, 2220.8496494431, 2223.8899020818, 2226.9307261489, 2229.9721209922, 2233.0140859607,
2236.0566204052, 2239.0997236778, 2242.1433951321, 2245.1876341232, 2248.2324400076, 2251.2778121433, 2254.3237498897, 2257.3702526077, 2260.4173196597, 2263.4649504095,
2266.5131442222, 2269.5619004644, 2272.6112185043, 2275.6610977113, 2278.7115374563, 2281.7625371116, 2284.8140960510, 2287.8662136494, 2290.9188892834, 2293.9721223309,
2297.0259121711, 2300.0802581848, 2303.1351597539, 2306.1906162618, 2309.2466270934, 2312.3031916347, 2315.3603092732, 2318.4179793978, 2321.4762013987, 2324.5349746674,
2327.5942985967, 2330.6541725810, 2333.7145960157, 2336.7755682977, 2339.8370888252, 2342.8991569977, 2345.9617722161, 2349.0249338824, 2352.0886414002, 2355.1528941741,
2358.2176916103, 2361.2830331159, 2364.3489180997, 2367.4153459716, 2370.4823161427, 2373.5498280256, 2376.6178810338, 2379.6864745825, 2382.7556080879, 2385.8252809675,
2388.8954926401, 2391.9662425258, 2395.0375300457, 2398.1093546225, 2401.1817156799, 2404.2546126428, 2407.3280449375, 2410.4020119915, 2413.4765132334, 2416.5515480932,
2419.6271160019, 2422.7032163918, 2425.7798486966, 2428.8570123509, 2431.9347067906, 2435.0129314529, 2438.0916857762, 2441.1709691999, 2444.2507811647, 2447.3311211125,
2450.4119884864, 2453.4933827306, 2456.5753032904, 2459.6577496125, 2462.7407211446, 2465.8242173356, 2468.9082376355, 2471.9927814956, 2475.0778483680, 2478.1634377064,
2481.2495489654, 2484.3361816008, 2487.4233350693, 2490.5110088292, 2493.5992023394, 2496.6879150604, 2499.7771464535, 2502.8668959812, 2505.9571631071, 2509.0479472961,
2512.1392480139, 2515.2310647274, 2518.3233969048, 2521.4162440152, 2524.5096055287, 2527.6034809168, 2530.6978696518, 2533.7927712072, 2536.8881850576, 2539.9841106787,
2543.0805475471, 2546.1774951407, 2549.2749529382, 2552.3729204197, 2555.4713970660, 2558.5703823593, 2561.6698757825, 2564.7698768199, 2567.8703849565, 2570.9713996787,
2574.0729204737, 2577.1749468298, 2580.2774782363, 2583.3805141836, 2586.4840541630, 2589.5880976671, 2592.6926441891, 2595.7976932237, 2598.9032442662, 2602.0092968131,
2605.1158503619, 2608.2229044112, 2611.3304584603, 2614.4385120099, 2617.5470645614, 2620.6561156173, 2623.7656646811, 2626.8757112572, 2629.9862548512, 2633.0972949694,
2636.2088311193, 2639.3208628092, 2642.4333895485, 2645.5464108477, 2648.6599262178, 2651.7739351713, 2654.8884372214, 2658.0034318823, 2661.1189186690, 2664.2348970978,
2667.3513666856, 2670.4683269504, 2673.5857774113, 2676.7037175880, 2679.8221470015, 2682.9410651734, 2686.0604716265, 2689.1803658844, 2692.3007474717, 2695.4216159138,
2698.5429707371, 2701.6648114691, 2704.7871376378, 2707.9099487725, 2711.0332444033, 2714.1570240611, 2717.2812872778, 2720.4060335862, 2723.5312625201, 2726.6569736139,
2729.7831664032, 2732.9098404244, 2736.0369952148, 2739.1646303125, 2742.2927452566, 2745.4213395869, 2748.5504128444, 2751.6799645708, 2754.8099943085, 2757.9405016011,
2761.0714859928, 2764.2029470289, 2767.3348842554, 2770.4672972192, 2773.6001854681, 2776.7335485508, 2779.8673860167, 2783.0016974162, 2786.1364823004, 2789.2717402215,
2792.4074707323, 2795.5436733865, 2798.6803477388, 2801.8174933446, 2804.9551097600, 2808.0931965423, 2811.2317532494, 2814.3707794399, 2817.5102746736, 2820.6502385107,
2823.7906705125, 2826.9315702411, 2830.0729372594, 2833.2147711310, 2836.3570714205, 2839.4998376931, 2842.6430695150, 2845.7867664531, 2848.9309280751, 2852.0755539497,
2855.2206436461, 2858.3661967344, 2861.5122127856, 2864.6586913715, 2867.8056320645, 2870.9530344379, 2874.1008980659, 2877.2492225233, 2880.3980073858, 2883.5472522299,
2886.6969566327, 2889.8471201723, 2892.9977424274, 2896.1488229775, 2899.3003614031, 2902.4523572851, 2905.6048102055, 2908.7577197468, 2911.9110854925, 2915.0649070266,
2918.2191839340, 2921.3739158004, 2924.5291022122, 2927.6847427566, 2930.8408370214, 2933.9973845953, 2937.1543850676, 2940.3118380286, 2943.4697430692, 2946.6280997808,
2949.7869077559, 2952.9461665876, 2956.1058758697, 2959.2660351967, 2962.4266441639, 2965.5877023674, 2968.7492094038, 2971.9111648707, 2975.0735683662, 2978.2364194892,
2981.3997178394, 2984.5634630170, 2987.7276546231, 2990.8922922595, 2994.0573755286, 2997.2229040337, 3000.3888773785, 3003.5552951677, 3006.7221570067, 3009.8894625012,
3013.0572112582, 3016.2254028848, 3019.3940369893, 3022.5631131804, 3025.7326310675, 3028.9025902608, 3032.0729903712, 3035.2438310102, 3038.4151117899, 3041.5868323234,
3044.7589922241, 3047.9315911063, 3051.1046285849, 3054.2781042756, 3057.4520177946, 3060.6263687588, 3063.8011567859, 3066.9763814942, 3070.1520425027, 3073.3281394308,
3076.5046718990, 3079.6816395281, 3082.8590419398, 3086.0368787563, 3089.2151496005, 3092.3938540960, 3095.5729918670, 3098.7525625384, 3101.9325657356, 3105.1130010850,
3108.2938682132, 3111.4751667478, 3114.6568963168, 3117.8390565489, 3121.0216470736, 3124.2046675209, 3127.3881175213, 3130.5719967063, 3133.7563047077, 3136.9410411580,
3140.1262056905, 3143.3117979390, 3146.4978175378, 3149.6842641220, 3152.8711373273, 3156.0584367901, 3159.2461621472, 3162.4343130361, 3165.6228890951, 3168.8118899628,
3172.0013152787, 3175.1911646828, 3178.3814378156, 3181.5721343184, 3184.7632538330, 3187.9547960018, 3191.1467604679, 3194.3391468748, 3197.5319548669, 3200.7251840889,
3203.9188341863, 3207.1129048050, 3210.3073955918, 3213.5023061938, 3216.6976362589, 3219.8933854354, 3223.0895533723, 3226.2861397192, 3229.4831441262, 3232.6805662442,
3235.8784057243, 3239.0766622186, 3242.2753353795, 3245.4744248602, 3248.6739303141, 3251.8738513956, 3255.0741877594, 3258.2749390609, 3261.4761049561, 3264.6776851015,
3267.8796791541, 3271.0820867715, 3274.2849076121, 3277.4881413345, 3280.6917875982, 3283.8958460629, 3287.1003163892, 3290.3051982381, 3293.5104912712, 3296.7161951505,
3299.9223095389, 3303.1288340995, 3306.3357684961, 3309.5431123930, 3312.7508654552, 3315.9590273481, 3319.1675977377, 3322.3765762906, 3325.5859626737, 3328.7957565547,
3332.0059576018, 3335.2165654837, 3338.4275798696, 3341.6390004293, 3344.8508268332, 3348.0630587519, 3351.2756958571, 3354.4887378204, 3357.7021843145, 3360.9160350123,
3364.1302895872, 3367.3449477134, 3370.5600090653, 3373.7754733180, 3376.9913401472, 3380.2076092289, 3383.4242802399, 3386.6413528573, 3389.8588267587, 3393.0767016224,
3396.2949771271, 3399.5136529520, 3402.7327287768, 3405.9522042819, 3409.1720791479, 3412.3923530562, 3415.6130256886, 3418.8340967272, 3422.0555658550, 3425.2774327553,
3428.4996971117, 3431.7223586087, 3434.9454169311, 3438.1688717641, 3441.3927227936, 3444.6169697059, 3447.8416121878, 3451.0666499265, 3454.2920826099, 3457.5179099263,
3460.7441315644, 3463.9707472134, 3467.1977565633, 3470.4251593041, 3473.6529551266, 3476.8811437221, 3480.1097247822, 3483.3386979991, 3486.5680630655, 3489.7978196746,
3493.0279675199, 3496.2585062956, 3499.4894356963, 3502.7207554169, 3505.9524651532, 3509.1845646010, 3512.4170534569, 3515.6499314178, 3518.8831981811, 3522.1168534448,
3525.3508969072, 3528.5853282672, 3531.8201472240, 3535.0553534775, 3538.2909467278, 3541.5269266757, 3544.7632930223, 3548.0000454692, 3551.2371837186, 3554.4747074730,
3557.7126164353, 3560.9509103091, 3564.1895887982, 3567.4286516071, 3570.6680984404, 3573.9079290036, 3577.1481430024, 3580.3887401428, 3583.6297201316, 3586.8710826759,
3590.1128274831, 3593.3549542612, 3596.5974627187, 3599.8403525644, 3603.0836235077, 3606.3272752582, 3609.5713075262, 3612.8157200222, 3616.0605124575, 3619.3056845434,
3622.5512359920, 3625.7971665156, 3629.0434758270, 3632.2901636395, 3635.5372296669, 3638.7846736231, 3642.0324952228, 3645.2806941810, 3648.5292702130, 3651.7782230347,
3655.0275523625, 3658.2772579129, 3661.5273394031, 3664.7777965507, 3668.0286290737, 3671.2798366903, 3674.5314191196, 3677.7833760806, 3681.0357072931, 3684.2884124772,
3687.5414913532, 3690.7949436423, 3694.0487690656, 3697.3029673450, 3700.5575382026, 3703.8124813610, 3707.0677965431, 3710.3234834725, 3713.5795418728, 3716.8359714683,
3720.0927719837, 3723.3499431439, 3726.6074846746, 3729.8653963013, 3733.1236777506, 3736.3823287490, 3739.6413490235, 3742.9007383017, 3746.1604963114, 3749.4206227810,
3752.6811174390, 3755.9419800146, 3759.2032102372, 3762.4648078367, 3765.7267725433, 3768.9891040878, 3772.2518022012, 3775.5148666149, 3778.7782970607, 3782.0420932710,
3785.3062549784, 3788.5707819158, 3791.8356738166, 3795.1009304148, 3798.3665514444, 3801.6325366400, 3804.8988857366, 3808.1655984695, 3811.4326745745, 3814.7001137877,
3817.9679158455, 3821.2360804849, 3824.5046074431, 3827.7734964578, 3831.0427472669, 3834.3123596089, 3837.5823332225, 3840.8526678469, 3844.1233632217, 3847.3944190867,
3850.6658351822, 3853.9376112489, 3857.2097470278, 3860.4822422602, 3863.7550966881, 3867.0283100535, 3870.3018820989, 3873.5758125672, 3876.8501012016, 3880.1247477459,
3883.3997519439, 3886.6751135400, 3889.9508322789, 3893.2269079057, 3896.5033401659, 3899.7801288053, 3903.0572735700, 3906.3347742065, 3909.6126304618, 3912.8908420830,
3916.1694088179, 3919.4483304144, 3922.7276066207, 3926.0072371857, 3929.2872218582, 3932.5675603878, 3935.8482525241, 3939.1292980173, 3942.4106966178, 3945.6924480764,
3948.9745521444, 3952.2570085730, 3955.5398171144, 3958.8229775206, 3962.1064895442, 3965.3903529381, 3968.6745674556, 3971.9591328503, 3975.2440488760, 3978.5293152872,
3981.8149318384, 3985.1008982845, 3988.3872143811, 3991.6738798835, 3994.9608945480, 3998.2482581308, 4001.5359703886, 4004.8240310784, 4008.1124399577, 4011.4011967840,
4014.6903013154, 4017.9797533103, 4021.2695525274, 4024.5596987257, 4027.8501916647, 4031.1410311039, 4034.4322168036, 4037.7237485239, 4041.0156260257, 4044.3078490700,
4047.6004174181, 4050.8933308318, 4054.1865890730, 4057.4801919041, 4060.7741390878, 4064.0684303871, 4067.3630655653, 4070.6580443861, 4073.9533666135, 4077.2490320118,
4080.5450403455, 4083.8413913797, 4087.1380848797, 4090.4351206109, 4093.7324983394, 4097.0302178314, 4100.3282788534, 4103.6266811723, 4106.9254245553, 4110.2245087699,
4113.5239335839, 4116.8236987655, 4120.1238040831, 4123.4242493055, 4126.7250342018, 4130.0261585414, 4133.3276220940, 4136.6294246296, 4139.9315659186, 4143.2340457316,
4146.5368638396, 4149.8400200138, 4153.1435140258, 4156.4473456475, 4159.7515146511, 4163.0560208090, 4166.3608638942, 4169.6660436797, 4172.9715599389, 4176.2774124456,
4179.5836009738, 4182.8901252977, 4186.1969851922, 4189.5041804320, 4192.8117107924, 4196.1195760489, 4199.4277759775, 4202.7363103542, 4206.0451789555, 4209.3543815581,
4212.6639179390, 4215.9737878756, 4219.2839911455, 4222.5945275267, 4225.9053967973, 4229.2165987359, 4232.5281331212, 4235.8399997325, 4239.1521983490, 4242.4647287506,
4245.7775907170, 4249.0907840288, 4252.4043084663, 4255.7181638105, 4259.0323498425, 4262.3468663437, 4265.6617130959, 4268.9768898811, 4272.2923964815, 4275.6082326799,
4278.9243982589, 4282.2408930019, 4285.5577166922, 4288.8748691136, 4292.1923500501, 4295.5101592859, 4298.8282966058, 4302.1467617944, 4305.4655546371, 4308.7846749191,
4312.1041224262, 4315.4238969445, 4318.7439982600, 4322.0644261596, 4325.3851804298, 4328.7062608579, 4332.0276672312, 4335.3493993373, 4338.6714569643, 4341.9938399003,
4345.3165479338, 4348.6395808536, 4351.9629384487, 4355.2866205083, 4358.6106268222, 4361.9349571801, 4365.2596113722, 4368.5845891889, 4371.9098904208, 4375.2355148589,
4378.5614622944, 4381.8877325188, 4385.2143253238, 4388.5412405014, 4391.8684778440, 4395.1960371440, 4398.5239181943, 4401.8521207880, 4405.1806447185, 4408.5094897793,
4411.8386557644, 4415.1681424678, 4418.4979496841, 4421.8280772078, 4425.1585248339, 4428.4892923576, 4431.8203795744, 4435.1517862801, 4438.4835122704, 4441.8155573418,
4445.1479212908, 4448.4806039140, 4451.8136050086, 4455.1469243717, 4458.4805618010, 4461.8145170942, 4465.1487900494, 4468.4833804649, 4471.8182881393, 4475.1535128713,
4478.4890544601, 4481.8249127050, 4485.1610874056, 4488.4975783617, 4491.8343853733, 4495.1715082410, 4498.5089467652, 4501.8467007467, 4505.1847699868, 4508.5231542867,
4511.8618534480, 4515.2008672727, 4518.5401955627, 4521.8798381205, 4525.2197947485, 4528.5600652498, 4531.9006494273, 4535.2415470845, 4538.5827580248, 4541.9242820521,
4545.2661189706, 4548.6082685844, 4551.9507306982, 4555.2935051168, 4558.6365916453, 4561.9799900889, 4565.3237002532, 4568.6677219439, 4572.0120549671, 4575.3566991291,
4578.7016542363, 4582.0469200955, 4585.3924965137, 4588.7383832981, 4592.0845802562, 4595.4310871957, 4598.7779039245, 4602.1250302508, 4605.4724659830, 4608.8202109298,
4612.1682649001, 4615.5166277029, 4618.8652991477, 4622.2142790441, 4625.5635672018, 4628.9131634310, 4632.2630675420, 4635.6132793452, 4638.9637986515, 4642.3146252719,
4645.6657590175, 4649.0171996999, 4652.3689471308, 4655.7210011220, 4659.0733614858, 4662.4260280344, 4665.7790005806, 4669.1322789372, 4672.4858629172, 4675.8397523340,
4679.1939470011, 4682.5484467322, 4685.9032513413, 4689.2583606427, 4692.6137744508, 4695.9694925802, 4699.3255148458, 4702.6818410628, 4706.0384710465, 4709.3954046124,
4712.7526415764, 4716.1101817544, 4719.4680249628, 4722.8261710178, 4726.1846197363, 4729.5433709351, 4732.9024244313, 4736.2617800423, 4739.6214375857, 4742.9813968792,
4746.3416577408, 4749.7022199888, 4753.0630834416, 4756.4242479179, 4759.7857132365, 4763.1474792165, 4766.5095456773, 4769.8719124384, 4773.2345793195, 4776.5975461405,
4779.9608127218, 4783.3243788836, 4786.6882444467, 4790.0524092317, 4793.4168730598, 4796.7816357521, 4800.1466971302, 4803.5120570158, 4806.8777152307, 4810.2436715971,
4813.6099259372, 4816.9764780736, 4820.3433278290, 4823.7104750264, 4827.0779194890, 4830.4456610401, 4833.8136995032, 4837.1820347023, 4840.5506664612, 4843.9195946043,
4847.2888189558, 4850.6583393405, 4854.0281555832, 4857.3982675089, 4860.7686749428, 4864.1393777104, 4867.5103756375, 4870.8816685497, 4874.2532562732, 4877.6251386344,
4880.9973154596, 4884.3697865755, 4887.7425518090, 4891.1156109873, 4894.4889639376, 4897.8626104874, 4901.2365504644, 4904.6107836966, 4907.9853100119, 4911.3601292388,
4914.7352412057, 4918.1106457414, 4921.4863426747, 4924.8623318347, 4928.2386130508, 4931.6151861525, 4934.9920509694, 4938.3692073315, 4941.7466550689, 4945.1243940118,
4948.5024239909, 4951.8807448367, 4955.2593563801, 4958.6382584523, 4962.0174508845, 4965.3969335083, 4968.7767061552, 4972.1567686572, 4975.5371208463, 4978.9177625548,
4982.2986936151, 4985.6799138599, 4989.0614231220, 4992.4432212345, 4995.8253080306, 4999.2076833437, 5002.5903470074, 5005.9732988555, 5009.3565387220, 5012.7400664412,
5016.1238818473, 5019.5079847750, 5022.8923750590, 5026.2770525342, 5029.6620170358, 5033.0472683991, 5036.4328064597, 5039.8186310532, 5043.2047420156, 5046.5911391828,
5049.9778223912, 5053.3647914773, 5056.7520462776, 5060.1395866290, 5063.5274123685, 5066.9155233333, 5070.3039193609, 5073.6926002887, 5077.0815659545, 5080.4708161963,
5083.8603508522, 5087.2501697605, 5090.6402727597, 5094.0306596884, 5097.4213303856, 5100.8122846903, 5104.2035224416, 5107.5950434791, 5110.9868476422, 5114.3789347708,
5117.7713047049, 5121.1639572844, 5124.5568923499, 5127.9501097416, 5131.3436093005, 5134.7373908672, 5138.1314542828, 5141.5257993885, 5144.9204260258, 5148.3153340361,
5151.7105232613, 5155.1059935432, 5158.5017447240, 5161.8977766459, 5165.2940891515, 5168.6906820832, 5172.0875552841, 5175.4847085969, 5178.8821418649, 5182.2798549315,
5185.6778476401, 5189.0761198344, 5192.4746713584, 5195.8735020559, 5199.2726117714, 5202.6720003490, 5206.0716676334, 5209.4716134694, 5212.8718377017, 5216.2723401756,
5219.6731207362, 5223.0741792290, 5226.4755154995, 5229.8771293936, 5233.2790207571, 5236.6811894362, 5240.0836352772, 5243.4863581264, 5246.8893578306, 5250.2926342365,
5253.6961871910, 5257.1000165413, 5260.5041221347, 5263.9085038186, 5267.3131614407, 5270.7180948488, 5274.1233038909, 5277.5287884150, 5280.9345482695, 5284.3405833028,
5287.7468933636, 5291.1534783008, 5294.5603379631, 5297.9674721999, 5301.3748808604, 5304.7825637940, 5308.1905208505, 5311.5987518795, 5315.0072567311, 5318.4160352553,
5321.8250873025, 5325.2344127231, 5328.6440113678, 5332.0538830873, 5335.4640277325, 5338.8744451545, 5342.2851352047, 5345.6960977345, 5349.1073325954, 5352.5188396391,
5355.9306187177, 5359.3426696832, 5362.7549923878, 5366.1675866838, 5369.5804524240, 5372.9935894610, 5376.4069976476, 5379.8206768369, 5383.2346268821, 5386.6488476365,
5390.0633389537, 5393.4781006873, 5396.8931326912, 5400.3084348193, 5403.7240069259, 5407.1398488651, 5410.5559604914, 5413.9723416596, 5417.3889922242, 5420.8059120403,
5424.2231009630, 5427.6405588475, 5431.0582855492, 5434.4762809236, 5437.8945448265, 5441.3130771136, 5444.7318776411, 5448.1509462652, 5451.5702828420, 5454.9898872281,
5458.4097592802, 5461.8298988550, 5465.2503058095, 5468.6709800007, 5472.0919212860, 5475.5131295226, 5478.9346045682, 5482.3563462805, 5485.7783545172, 5489.2006291365,
5492.6231699964, 5496.0459769554, 5499.4690498717, 5502.8923886041, 5506.3159930113, 5509.7398629522, 5513.1639982859, 5516.5883988715, 5520.0130645685, 5523.4379952363,
5526.8631907346, 5530.2886509233, 5533.7143756621, 5537.1403648114, 5540.5666182312, 5543.9931357820, 5547.4199173244, 5550.8469627190, 5554.2742718266, 5557.7018445083,
5561.1296806252, 5564.5577800386, 5567.9861426098, 5571.4147682005, 5574.8436566724, 5578.2728078873, 5581.7022217072, 5585.1318979943, 5588.5618366108, 5591.9920374193,
5595.4225002823, 5598.8532250625, 5602.2842116228, 5605.7154598261, 5609.1469695358, 5612.5787406150, 5616.0107729273, 5619.4430663361, 5622.8756207052, 5626.3084358986,
5629.7415117801, 5633.1748482140, 5636.6084450646, 5640.0423021962, 5643.4764194735, 5646.9107967613, 5650.3454339242, 5653.7803308274, 5657.2154873360, 5660.6509033153,
5664.0865786306, 5667.5225131476, 5670.9587067320, 5674.3951592495, 5677.8318705662, 5681.2688405481, 5684.7060690616, 5688.1435559730, 5691.5813011489, 5695.0193044559,
5698.4575657608, 5701.8960849306, 5705.3348618323, 5708.7738963332, 5712.2131883006, 5715.6527376020, 5719.0925441050, 5722.5326076775, 5725.9729281872, 5729.4135055023,
5732.8543394909, 5736.2954300212, 5739.7367769619, 5743.1783801813, 5746.6202395483, 5750.0623549316, 5753.5047262003, 5756.9473532235, 5760.3902358704, 5763.8333740103,
5767.2767675129, 5770.7204162477, 5774.1643200846, 5777.6084788933, 5781.0528925441, 5784.4975609071, 5787.9424838525, 5791.3876612508, 5794.8330929726, 5798.2787788886,
5801.7247188697, 5805.1709127867, 5808.6173605108, 5812.0640619133, 5815.5110168654, 5818.9582252388, 5822.4056869049, 5825.8534017355, 5829.3013696026, 5832.7495903782,
5836.1980639343, 5839.6467901432, 5843.0957688774, 5846.5450000093, 5849.9944834117, 5853.4442189573, 5856.8942065190, 5860.3444459699, 5863.7949371830, 5867.2456800318,
5870.6966743897, 5874.1479201301, 5877.5994171269, 5881.0511652536, 5884.5031643844, 5887.9554143933, 5891.4079151544, 5894.8606665421, 5898.3136684307, 5901.7669206949,
5905.2204232094, 5908.6741758489, 5912.1281784883, 5915.5824310029, 5919.0369332677, 5922.4916851580, 5925.9466865493, 5929.4019373172, 5932.8574373373, 5936.3131864854,
5939.7691846375, 5943.2254316697, 5946.6819274580, 5950.1386718789, 5953.5956648087, 5957.0529061240, 5960.5103957014, 5963.9681334178, 5967.4261191500, 5970.8843527752,
5974.3428341704, 5977.8015632129, 5981.2605397802, 5984.7197637498, 5988.1792349992, 5991.6389534064, 5995.0989188491, 5998.5591312054, 6002.0195903533, 6005.4802961712,
6008.9412485375, 6012.4024473305, 6015.8638924290, 6019.3255837115, 6022.7875210571, 6026.2497043446, 6029.7121334531, 6033.1748082619, 6036.6377286502, 6040.1008944976,
6043.5643056835, 6047.0279620876, 6050.4918635898, 6053.9560100698, 6057.4204014079, 6060.8850374841, 6064.3499181787, 6067.8150433720, 6071.2804129446, 6074.7460267771,
6078.2118847502, 6081.6779867448, 6085.1443326418, 6088.6109223223, 6092.0777556675, 6095.5448325588, 6099.0121528775, 6102.4797165051, 6105.9475233235, 6109.4155732142,
6112.8838660593, 6116.3524017407, 6119.8211801405, 6123.2902011410, 6126.7594646246, 6130.2289704735, 6133.6987185706, 6137.1687087984, 6140.6389410397, 6144.1094151775,
6147.5801310948, 6151.0510886747, 6154.5222878005, 6157.9937283555, 6161.4654102233, 6164.9373332874, 6168.4094974315, 6171.8819025395, 6175.3545484953, 6178.8274351828,
6182.3005624864, 6185.7739302901, 6189.2475384785, 6192.7213869360, 6196.1954755471, 6199.6698041967, 6203.1443727694, 6206.6191811503, 6210.0942292244, 6213.5695168768,
6217.0450439929, 6220.5208104578, 6223.9968161573, 6227.4730609767, 6230.9495448019, 6234.4262675186, 6237.9032290128, 6241.3804291704, 6244.8578678776, 6248.3355450207,
6251.8134604860, 6255.2916141600, 6258.7700059292, 6262.2486356803, 6265.7275033001, 6269.2066086754, 6272.6859516934, 6276.1655322410, 6279.6453502055, 6283.1254054742,
6286.6056979346, 6290.0862274741, 6293.5669939805, 6297.0479973414, 6300.5292374448, 6304.0107141785, 6307.4924274306, 6310.9743770894, 6314.4565630431, 6317.9389851800,
6321.4216433887, 6324.9045375577, 6328.3876675758, 6331.8710333317, 6335.3546347144, 6338.8384716128, 6342.3225439162, 6345.8068515136, 6349.2913942945, 6352.7761721483,
6356.2611849644, 6359.7464326326, 6363.2319150426, 6366.7176320841, 6370.2035836473, 6373.6897696221, 6377.1761898986, 6380.6628443672, 6384.1497329181, 6387.6368554419,
6391.1242118291, 6394.6118019704, 6398.0996257566, 6401.5876830785, 6405.0759738270, 6408.5644978934, 6412.0532551687, 6415.5422455442, 6419.0314689113, 6422.5209251615,
6426.0106141864, 6429.5005358776, 6432.9906901269, 6436.4810768263, 6439.9716958677, 6443.4625471431, 6446.9536305448, 6450.4449459651, 6453.9364932963, 6457.4282724309,
6460.9202832615, 6464.4125256809, 6467.9049995817, 6471.3977048568, 6474.8906413993, 6478.3838091022, 6481.8772078587, 6485.3708375621, 6488.8646981057, 6492.3587893831,
6495.8531112877, 6499.3476637133, 6502.8424465537, 6506.3374597026, 6509.8327030542, 6513.3281765023, 6516.8238799413, 6520.3198132653, 6523.8159763687, 6527.3123691460,
6530.8089914916, 6534.3058433004, 6537.8029244669, 6541.3002348860, 6544.7977744527, 6548.2955430620, 6551.7935406090, 6555.2917669890, 6558.7902220972, 6562.2889058291,
6565.7878180802, 6569.2869587460, 6572.7863277224, 6576.2859249051, 6579.7857501899, 6583.2858034729, 6586.7860846502, 6590.2865936179, 6593.7873302723, 6597.2882945097,
6600.7894862267, 6604.2909053197, 6607.7925516855, 6611.2944252207, 6614.7965258222, 6618.2988533869, 6621.8014078119, 6625.3041889942, 6628.8071968311, 6632.3104312199,
6635.8138920580, 6639.3175792428, 6642.8214926719, 6646.3256322431, 6649.8299978540, 6653.3345894026, 6656.8394067867, 6660.3444499045, 6663.8497186540, 6667.3552129336,
6670.8609326414, 6674.3668776759, 6677.8730479357, 6681.3794433193, 6684.8860637253, 6688.3929090527, 6691.8999792001, 6695.4072740667, 6698.9147935514, 6702.4225375534,
6705.9305059719, 6709.4386987062, 6712.9471156558, 6716.4557567201, 6719.9646217987, 6723.4737107914, 6726.9830235978, 6730.4925601179, 6734.0023202515, 6737.5123038988,
6741.0225109598, 6744.5329413348, 6748.0435949241, 6751.5544716281, 6755.0655713472, 6758.5768939820, 6762.0884394333, 6765.6002076017, 6769.1121983880, 6772.6244116933,
6776.1368474185, 6779.6495054647, 6783.1623857331, 6786.6754881250, 6790.1888125418, 6793.7023588849, 6797.2161270559, 6800.7301169563, 6804.2443284879, 6807.7587615525,
6811.2734160521, 6814.7882918884, 6818.3033889638, 6821.8187071802, 6825.3342464399, 6828.8500066452, 6832.3659876986, 6835.8821895026, 6839.3986119597, 6842.9152549725,
6846.4321184440, 6849.9492022768, 6853.4665063739, 6856.9840306383, 6860.5017749732, 6864.0197392816, 6867.5379234669, 6871.0563274324, 6874.5749510816, 6878.0937943179,
6881.6128570450, 6885.1321391666, 6888.6516405864, 6892.1713612083, 6895.6913009363, 6899.2114596743, 6902.7318373265, 6906.2524337971, 6909.7732489903, 6913.2942828105,
6916.8155351622, 6920.3370059499, 6923.8586950782, 6927.3806024518, 6930.9027279755, 6934.4250715541, 6937.9476330926, 6941.4704124961, 6944.9934096695, 6948.5166245182,
6952.0400569474, 6955.5637068624, 6959.0875741687, 6962.6116587719, 6966.1359605774, 6969.6604794911, 6973.1852154187, 6976.7101682660, 6980.2353379390, 6983.7607243436,
6987.2863273860, 6990.8121469724, 6994.3381830090, 6997.8644354021, 7001.3909040582, 7004.9175888838, 7008.4444897854, 7011.9716066698, 7015.4989394436, 7019.0264880136,
7022.5542522869, 7026.0822321703, 7029.6104275710, 7033.1388383960, 7036.6674645527, 7040.1963059482, 7043.7253624901, 7047.2546340857, 7050.7841206426, 7054.3138220685,
7057.8437382709, 7061.3738691578, 7064.9042146369, 7068.4347746163, 7071.9655490038, 7075.4965377077, 7079.0277406361, 7082.5591576973, 7086.0907887995, 7089.6226338512,
7093.1546927610, 7096.6869654373, 7100.2194517888, 7103.7521517242, 7107.2850651524, 7110.8181919821, 7114.3515321225, 7117.8850854825, 7121.4188519711, 7124.9528314977,
7128.4870239715, 7132.0214293018, 7135.5560473981, 7139.0908781698, 7142.6259215266, 7146.1611773780, 7149.6966456338, 7153.2323262039, 7156.7682189980, 7160.3043239263,
7163.8406408986, 7167.3771698251, 7170.9139106160, 7174.4508631816, 7177.9880274322, 7181.5254032783, 7185.0629906302, 7188.6007893986, 7192.1387994942, 7195.6770208276,
7199.2154533096, 7202.7540968512, 7206.2929513632, 7209.8320167567, 7213.3712929428, 7216.9107798326, 7220.4504773374, 7223.9903853685, 7227.5305038373, 7231.0708326553,
7234.6113717340, 7238.1521209850, 7241.6930803201, 7245.2342496509, 7248.7756288894, 7252.3172179475, 7255.8590167371, 7259.4010251702, 7262.9432431592, 7266.4856706160,
7270.0283074531, 7273.5711535828, 7277.1142089175, 7280.6574733697, 7284.2009468520, 7287.7446292770, 7291.2885205575, 7294.8326206063, 7298.3769293362, 7301.9214466602,
7305.4661724913, 7309.0111067425, 7312.5562493270, 7316.1016001581, 7319.6471591491, 7323.1929262133, 7326.7389012642, 7330.2850842153, 7333.8314749802, 7337.3780734725,
7340.9248796060, 7344.4718932946, 7348.0191144520, 7351.5665429922, 7355.1141788293, 7358.6620218773, 7362.2100720504, 7365.7583292628, 7369.3067934288, 7372.8554644629,
7376.4043422795, 7379.9534267930, 7383.5027179181, 7387.0522155694, 7390.6019196616, 7394.1518301096, 7397.7019468282, 7401.2522697324, 7404.8027987372, 7408.3535337576,
7411.9044747088, 7415.4556215060, 7419.0069740646, 7422.5585322998, 7426.1102961271, 7429.6622654620, 7433.2144402201, 7436.7668203170, 7440.3194056684, 7443.8721961900,
7447.4251917978, 7450.9783924077, 7454.5317979356, 7458.0854082975, 7461.6392234097, 7465.1932431882, 7468.7474675494, 7472.3018964096, 7475.8565296851, 7479.4113672925,
7482.9664091482, 7486.5216551688, 7490.0771052711, 7493.6327593718, 7497.1886173876, 7500.7446792355, 7504.3009448324, 7507.8574140952, 7511.4140869411, 7514.9709632871,
7518.5280430506, 7522.0853261488, 7525.6428124990, 7529.2005020186, 7532.7583946252, 7536.3164902362, 7539.8747887693, 7543.4332901421, 7546.9919942724, 7550.5509010781,
7554.1100104769, 7557.6693223869, 7561.2288367260, 7564.7885534123, 7568.3484723640, 7571.9085934993, 7575.4689167364, 7579.0294419937, 7582.5901691896, 7586.1510982426,
7589.7122290712, 7593.2735615941, 7596.8350957298, 7600.3968313972, 7603.9587685151, 7607.5209070022, 7611.0832467775, 7614.6457877601, 7618.2085298691, 7621.7714730234,
7625.3346171424, 7628.8979621452, 7632.4615079513, 7636.0252544799, 7639.5892016506, 7643.1533493829, 7646.7176975963, 7650.2822462105, 7653.8469951452, 7657.4119443202,
7660.9770936553, 7664.5424430705, 7668.1079924856, 7671.6737418208, 7675.2396909961, 7678.8058399317, 7682.3721885478, 7685.9387367646, 7689.5054845027, 7693.0724316822,
7696.6395782238, 7700.2069240480, 7703.7744690754, 7707.3422132266, 7710.9101564224, 7714.4782985836, 7718.0466396310, 7721.6151794856, 7725.1839180683, 7728.7528553002,
7732.3219911023, 7735.8913253960, 7739.4608581023, 7743.0305891426, 7746.6005184382, 7750.1706459107, 7753.7409714813, 7757.3114950718, 7760.8822166037, 7764.4531359987,
7768.0242531784, 7771.5955680648, 7775.1670805796, 7778.7387906447, 7782.3106981823, 7785.8828031142, 7789.4551053626, 7793.0276048497, 7796.6003014976, 7800.1731952287,
7803.7462859653, 7807.3195736298, 7810.8930581448, 7814.4667394326, 7818.0406174159, 7821.6146920175, 7825.1889631598, 7828.7634307659, 7832.3380947584, 7835.9129550603,
7839.4880115945, 7843.0632642841, 7846.6387130521, 7850.2143578217, 7853.7901985160, 7857.3662350584, 7860.9424673721, 7864.5188953805, 7868.0955190070, 7871.6723381752,
7875.2493528086, 7878.8265628308, 7882.4039681655, 7885.9815687364, 7889.5593644674, 7893.1373552822, 7896.7155411049, 7900.2939218593, 7903.8724974695, 7907.4512678596,
7911.0302329537, 7914.6093926761, 7918.1887469511, 7921.7682957028, 7925.3480388559, 7928.9279763346, 7932.5081080635, 7936.0884339671, 7939.6689539701, 7943.2496679972,
7946.8305759731, 7950.4116778225, 7953.9929734704, 7957.5744628417, 7961.1561458613, 7964.7380224542, 7968.3200925456, 7971.9023560606, 7975.4848129244, 7979.0674630622,
7982.6503063994, 7986.2333428614, 7989.8165723736, 7993.3999948614, 7996.9836102505, 8000.5674184664, 8004.1514194348, 8007.7356130814, 8011.3199993321, 8014.9045781125,
8018.4893493487, 8022.0743129666, 8025.6594688922, 8029.2448170515, 8032.8303573707, 8036.4160897759, 8040.0020141935, 8043.5881305496, 8047.1744387706, 8050.7609387829,
8054.3476305131, 8057.9345138875, 8061.5215888328, 8065.1088552757, 8068.6963131427, 8072.2839623606, 8075.8718028563, 8079.4598345566, 8083.0480573883, 8086.6364712785,
8090.2250761542, 8093.8138719425, 8097.4028585704, 8100.9920359652, 8104.5814040542, 8108.1709627646, 8111.7607120237, 8115.3506517590, 8118.9407818980, 8122.5311023681,
8126.1216130970, 8129.7123140123, 8133.3032050416, 8136.8942861128, 8140.4855571535, 8144.0770180917, 8147.6686688553, 8151.2605093721, 8154.8525395703, 8158.4447593779,
8162.0371687231, 8165.6297675339, 8169.2225557386, 8172.8155332656, 8176.4087000432, 8180.0020559997, 8183.5956010636, 8187.1893351634, 8190.7832582277, 8194.3773701851,
8197.9716709642, 8201.5661604938, 8205.1608387026, 8208.7557055195, 8212.3507608733, 8215.9460046931, 8219.5414369077, 8223.1370574462, 8226.7328662377, 8230.3288632114,
8233.9250482964, 8237.5214214221, 8241.1179825176, 8244.7147315125, 8248.3116683360, 8251.9087929177, 8255.5061051871, 8259.1036050738, 8262.7012925073, 8266.2991674174,
8269.8972297338, 8273.4954793863, 8277.0939163046, 8280.6925404188, 8284.2913516587, 8287.8903499544, 8291.4895352358, 8295.0889074332, 8298.6884664765, 8302.2882122961,
8305.8881448222, 8309.4882639851, 8313.0885697152, 8316.6890619428, 8320.2897405984, 8323.8906056126, 8327.4916569160, 8331.0928944390, 8334.6943181125, 8338.2959278671,
8341.8977236336, 8345.4997053428, 8349.1018729256, 8352.7042263130, 8356.3067654358, 8359.9094902252, 8363.5124006122, 8367.1154965280, 8370.7187779037, 8374.3222446706,
8377.9258967599, 8381.5297341030, 8385.1337566313, 8388.7379642763, 8392.3423569693, 8395.9469346420, 8399.5516972259, 8403.1566446527, 8406.7617768541, 8410.3670937618,
8413.9725953075, 8417.5782814233, 8421.1841520409, 8424.7902070922, 8428.3964465094, 8432.0028702243, 8435.6094781692, 8439.2162702761, 8442.8232464773, 8446.4304067050,
8450.0377508915, 8453.6452789691, 8457.2529908702, 8460.8608865273, 8464.4689658728, 8468.0772288394, 8471.6856753596, 8475.2943053659, 8478.9031187913, 8482.5121155682,
8486.1212956297, 8489.7306589084, 8493.3402053373, 8496.9499348493, 8500.5598473774, 8504.1699428547, 8507.7802212141, 8511.3906823890, 8515.0013263123, 8518.6121529174,
8522.2231621376, 8525.8343539061, 8529.4457281564, 8533.0572848219, 8536.6690238360, 8540.2809451323, 8543.8930486443, 8547.5053343056, 8551.1178020500, 8554.7304518112,
8558.3432835228, 8561.9562971188, 8565.5694925329, 8569.1828696991, 8572.7964285514, 8576.4101690237, 8580.0240910501, 8583.6381945648, 8587.2524795017, 8590.8669457952,
8594.4815933795, 8598.0964221889, 8601.7114321577, 8605.3266232202, 8608.9419953111, 8612.5575483646, 8616.1732823154, 8619.7891970980, 8623.4052926471, 8627.0215688972,
8630.6380257832, 8634.2546632398, 8637.8714812019, 8641.4884796041, 8645.1056583816, 8648.7230174692, 8652.3405568019, 8655.9582763148, 8659.5761759430, 8663.1942556216,
8666.8125152857, 8670.4309548708, 8674.0495743119, 8677.6683735444, 8681.2873525038, 8684.9065111254, 8688.5258493447, 8692.1453670972, 8695.7650643185, 8699.3849409441,
8703.0049969098, 8706.6252321512, 8710.2456466040, 8713.8662402042, 8717.4870128874, 8721.1079645895, 8724.7290952466, 8728.3504047946, 8731.9718931694, 8735.5935603072,
8739.2154061441, 8742.8374306162, 8746.4596336597, 8750.0820152109, 8753.7045752060, 8757.3273135815, 8760.9502302736, 8764.5733252189, 8768.1965983537, 8771.8200496147,
8775.4436789383, 8779.0674862612, 8782.6914715201, 8786.3156346516, 8789.9399755924, 8793.5644942795, 8797.1891906496, 8800.8140646395, 8804.4391161863, 8808.0643452268,
8811.6897516982, 8815.3153355374, 8818.9410966815, 8822.5670350678, 8826.1931506334, 8829.8194433155, 8833.4459130515, 8837.0725597785, 8840.6993834341, 8844.3263839557,
8847.9535612806, 8851.5809153464, 8855.2084460907, 8858.8361534510, 8862.4640373649, 8866.0920977702, 8869.7203346046, 8873.3487478058, 8876.9773373117, 8880.6061030601,
8884.2350449889, 8887.8641630360, 8891.4934571395, 8895.1229272373, 8898.7525732676, 8902.3823951684, 8906.0123928779, 8909.6425663344, 8913.2729154760, 8916.9034402411,
8920.5341405680, 8924.1650163950, 8927.7960676607, 8931.4272943034, 8935.0586962617, 8938.6902734740, 8942.3220258791, 8945.9539534156, 8949.5860560220, 8953.2183336372,
8956.8507862000, 8960.4834136490, 8964.1162159232, 8967.7491929615, 8971.3823447028, 8975.0156710860, 8978.6491720503, 8982.2828475346, 8985.9166974781, 8989.5507218200,
8993.1849204993, 8996.8192934554, 9000.4538406275, 9004.0885619549, 9007.7234573771, 9011.3585268334, 9014.9937702632, 9018.6291876060, 9022.2647788015, 9025.9005437890,
9029.5364825083, 9033.1725948991, 9036.8088809009, 9040.4453404536, 9044.0819734969, 9047.7187799707, 9051.3557598148, 9054.9929129691, 9058.6302393735, 9062.2677389682,
9065.9054116930, 9069.5432574881, 9073.1812762935, 9076.8194680495, 9080.4578326962, 9084.0963701739, 9087.7350804229, 9091.3739633834, 9095.0130189958, 9098.6522472006,
9102.2916479381, 9105.9312211490, 9109.5709667736, 9113.2108847525, 9116.8509750264, 9120.4912375360, 9124.1316722218, 9127.7722790247, 9131.4130578854, 9135.0540087447,
9138.6951315435, 9142.3364262226, 9145.9778927231, 9149.6195309858, 9153.2613409519, 9156.9033225622, 9160.5454757580, 9164.1878004804, 9167.8302966706, 9171.4729642698,
9175.1158032191, 9178.7588134601, 9182.4019949338, 9186.0453475819, 9189.6888713456, 9193.3325661664, 9196.9764319859, 9200.6204687455, 9204.2646763869, 9207.9090548517,
9211.5536040814, 9215.1983240179, 9218.8432146029, 9222.4882757781, 9226.1335074853, 9229.7789096665, 9233.4244822635, 9237.0702252182, 9240.7161384727, 9244.3622219688,
9248.0084756488, 9251.6548994547, 9255.3014933286, 9258.9482572127, 9262.5951910492, 9266.2422947804, 9269.8895683485, 9273.5370116959, 9277.1846247650, 9280.8324074981,
9284.4803598377, 9288.1284817263, 9291.7767731065, 9295.4252339207, 9299.0738641116, 9302.7226636219, 9306.3716323942, 9310.0207703712, 9313.6700774957, 9317.3195537105,
9320.9691989584, 9324.6190131823, 9328.2689963251, 9331.9191483299, 9335.5694691394, 9339.2199586969, 9342.8706169453, 9346.5214438277, 9350.1724392874, 9353.8236032675,
9357.4749357111, 9361.1264365617, 9364.7781057624, 9368.4299432566, 9372.0819489877, 9375.7341228990, 9379.3864649341, 9383.0389750364, 9386.6916531495, 9390.3444992169,
9393.9975131822, 9397.6506949890, 9401.3040445811, 9404.9575619022, 9408.6112468959, 9412.2650995061, 9415.9191196767, 9419.5733073514, 9423.2276624742, 9426.8821849890,
9430.5368748399, 9434.1917319708, 9437.8467563257, 9441.5019478488, 9445.1573064842, 9448.8128321761, 9452.4685248687, 9456.1243845061, 9459.7804110327, 9463.4366043929,
9467.0929645308, 9470.7494913911, 9474.4061849179, 9478.0630450559, 9481.7200717496, 9485.3772649434, 9489.0346245819, 9492.6921506098, 9496.3498429716, 9500.0077016122,
9503.6657264762, 9507.3239175083, 9510.9822746533, 9514.6407978562, 9518.2994870617, 9521.9583422148, 9525.6173632603, 9529.2765501434, 9532.9359028089, 9536.5954212020,
9540.2551052677, 9543.9149549511, 9547.5749701975, 9551.2351509519, 9554.8954971597, 9558.5560087662, 9562.2166857165, 9565.8775279561, 9569.5385354303, 9573.1997080845,
9576.8610458643, 9580.5225487150, 9584.1842165822, 9587.8460494115, 9591.5080471484, 9595.1702097386, 9598.8325371278, 9602.4950292616, 9606.1576860857, 9609.8205075460,
9613.4834935883, 9617.1466441584, 9620.8099592022, 9624.4734386655, 9628.1370824944, 9631.8008906349, 9635.4648630328, 9639.1289996344, 9642.7933003857, 9646.4577652329,
9650.1223941220, 9653.7871869993, 9657.4521438110, 9661.1172645034, 9664.7825490229, 9668.4479973156, 9672.1136093281, 9675.7793850067, 9679.4453242978, 9683.1114271480,
9686.7776935037, 9690.4441233116, 9694.1107165181, 9697.7774730700, 9701.4443929138, 9705.1114759962, 9708.7787222641, 9712.4461316641, 9716.1137041430, 9719.7814396476,
9723.4493381249, 9727.1173995217, 9730.7856237849, 9734.4540108616, 9738.1225606986, 9741.7912732431, 9745.4601484422, 9749.1291862428, 9752.7983865922, 9756.4677494376,
9760.1372747261, 9763.8069624049, 9767.4768124215, 9771.1468247230, 9774.8169992568, 9778.4873359703, 9782.1578348109, 9785.8284957261, 9789.4993186633, 9793.1703035700,
9796.8414503939, 9800.5127590824, 9804.1842295832, 9807.8558618439, 9811.5276558123, 9815.1996114361, 9818.8717286629, 9822.5440074406, 9826.2164477171, 9829.8890494400,
9833.5618125575, 9837.2347370173, 9840.9078227674, 9844.5810697558, 9848.2544779306, 9851.9280472397, 9855.6017776313, 9859.2756690535, 9862.9497214545, 9866.6239347823,
9870.2983089854, 9873.9728440118, 9877.6475398100, 9881.3223963282, 9884.9974135147, 9888.6725913181, 9892.3479296866, 9896.0234285687, 9899.6990879130, 9903.3749076679,
9907.0508877821, 9910.7270282040, 9914.4033288823, 9918.0797897657, 9921.7564108028, 9925.4331919424, 9929.1101331332, 9932.7872343240, 9936.4644954636, 9940.1419165009,
9943.8194973847, 9947.4972380640, 9951.1751384877, 9954.8531986048, 9958.5314183643, 9962.2097977152, 9965.8883366066, 9969.5670349877, 9973.2458928075, 9976.9249100153,
9980.6040865603, 9984.2834223916, 9987.9629174586, 9991.6425717106, 9995.3223850969, 9999.0023575668, 10002.6824890699, 10006.3627795554, 10010.0432289729, 10013.7238372719,
10017.4046044019, 10021.0855303124, 10024.7666149532, 10028.4478582736, 10032.1292602236, 10035.8108207526, 10039.4925398105, 10043.1744173470, 10046.8564533119, 10050.5386476550,
10054.2210003262, 10057.9035112753, 10061.5861804522, 10065.2690078069, 10068.9519932894, 10072.6351368497, 10076.3184384378, 10080.0018980037, 10083.6855154976, 10087.3692908697,
10091.0532240700, 10094.7373150488, 10098.4215637563, 10102.1059701428, 10105.7905341585, 10109.4752557538, 10113.1601348790, 10116.8451714846, 10120.5303655208, 10124.2157169383,
10127.9012256874, 10131.5868917186, 10135.2727149826, 10138.9586954298, 10142.6448330109, 10146.3311276766, 10150.0175793773, 10153.7041880640, 10157.3909536872, 10161.0778761979,
10164.7649555466, 10168.4521916843, 10172.1395845618, 10175.8271341300, 10179.5148403398, 10183.2027031422, 10186.8907224880, 10190.5788983284, 10194.2672306144, 10197.9557192969,
10201.6443643272, 10205.3331656563, 10209.0221232354, 10212.7112370157, 10216.4005069484, 10220.0899329848, 10223.7795150761, 10227.4692531736, 10231.1591472287, 10234.8491971927,
10238.5394030171, 10242.2297646533, 10245.9202820527, 10249.6109551668, 10253.3017839472, 10256.9927683453, 10260.6839083129, 10264.3752038015, 10268.0666547626, 10271.7582611481,
10275.4500229096, 10279.1419399989, 10282.8340123676, 10286.5262399676, 10290.2186227508, 10293.9111606689, 10297.6038536739, 10301.2967017176, 10304.9897047520, 10308.6828627291,
10312.3761756008, 10316.0696433193, 10319.7632658365, 10323.4570431045, 10327.1509750755, 10330.8450617017, 10334.5393029351, 10338.2336987281, 10341.9282490327, 10345.6229538014,
10349.3178129864, 10353.0128265400, 10356.7079944146, 10360.4033165626, 10364.0987929364, 10367.7944234884, 10371.4902081711, 10375.1861469369, 10378.8822397386, 10382.5784865285,
10386.2748872593, 10389.9714418836, 10393.6681503541, 10397.3650126233, 10401.0620286442, 10404.7591983693, 10408.4565217514, 10412.1539987433, 10415.8516292979, 10419.5494133681,
10423.2473509065, 10426.9454418663, 10430.6436862003, 10434.3420838616, 10438.0406348030, 10441.7393389776, 10445.4381963385, 10449.1372068387, 10452.8363704315, 10456.5356870698,
10460.2351567069, 10463.9347792960, 10467.6345547902, 10471.3344831429, 10475.0345643074, 10478.7347982369, 10482.4351848847, 10486.1357242044, 10489.8364161492, 10493.5372606725,
10497.2382577279, 10500.9394072688, 10504.6407092487, 10508.3421636212, 10512.0437703398, 10515.7455293581, 10519.4474406298, 10523.1495041085, 10526.8517197479, 10530.5540875016,
10534.2566073235, 10537.9592791672, 10541.6621029867, 10545.3650787356, 10549.0682063679, 10552.7714858374, 10556.4749170981, 10560.1785001038, 10563.8822348086, 10567.5861211663,
10571.2901591311, 10574.9943486570, 10578.6986896980, 10582.4031822083, 10586.1078261419, 10589.8126214531, 10593.5175680960, 10597.2226660248, 10600.9279151938, 10604.6333155573,
10608.3388670694, 10612.0445696846, 10615.7504233572, 10619.4564280416, 10623.1625836922, 10626.8688902634, 10630.5753477096, 10634.2819559854, 10637.9887150453, 10641.6956248438,
10645.4026853355, 10649.1098964750, 10652.8172582168, 10656.5247705158, 10660.2324333265, 10663.9402466036, 10667.6482103019, 10671.3563243761, 10675.0645887811, 10678.7730034716,
10682.4815684024, 10686.1902835286, 10689.8991488049, 10693.6081641862, 10697.3173296276, 10701.0266450840, 10704.7361105103, 10708.4457258618, 10712.1554910933, 10715.8654061599,
10719.5754710169, 10723.2856856193, 10726.9960499223, 10730.7065638811, 10734.4172274509, 10738.1280405870, 10741.8390032446, 10745.5501153790, 10749.2613769456, 10752.9727878996,
10756.6843481966, 10760.3960577919, 10764.1079166409, 10767.8199246990, 10771.5320819219, 10775.2443882650, 10778.9568436837, 10782.6694481338, 10786.3822015708, 10790.0951039502,
10793.8081552278, 10797.5213553593, 10801.2347043002, 10804.9482020064, 10808.6618484336, 10812.3756435376, 10816.0895872742, 10819.8036795991, 10823.5179204684, 10827.2323098377,
10830.9468476632, 10834.6615339006, 10838.3763685060, 10842.0913514353, 10845.8064826445, 10849.5217620898, 10853.2371897271, 10856.9527655125, 10860.6684894022, 10864.3843613523,
10868.1003813190, 10871.8165492585, 10875.5328651269, 10879.2493288806, 10882.9659404758, 10886.6826998689, 10890.3996070160, 10894.1166618736, 10897.8338643981, 10901.5512145459,
10905.2687122734, 10908.9863575369, 10912.7041502932, 10916.4220904985, 10920.1401781095, 10923.8584130827, 10927.5767953748, 10931.2953249422, 10935.0140017416, 10938.7328257298,
10942.4517968633, 10946.1709150989, 10949.8901803933, 10953.6095927034, 10957.3291519857, 10961.0488581973, 10964.7687112949, 10968.4887112353, 10972.2088579755, 10975.9291514724,
10979.6495916829, 10983.3701785640, 10987.0909120726, 10990.8117921659, 10994.5328188007, 10998.2539919342, 11001.9753115235, 11005.6967775256, 11009.4183898978, 11013.1401485972,
11016.8620535809, 11020.5841048063, 11024.3063022304, 11028.0286458107, 11031.7511355043, 11035.4737712687, 11039.1965530610, 11042.9194808388, 11046.6425545594, 11050.3657741801,
11054.0891396585, 11057.8126509520, 11061.5363080181, 11065.2601108143, 11068.9840592982, 11072.7081534272, 11076.4323931591, 11080.1567784513, 11083.8813092616, 11087.6059855476,
11091.3308072670, 11095.0557743775, 11098.7808868368, 11102.5061446027, 11106.2315476330, 11109.9570958855, 11113.6827893180, 11117.4086278884, 11121.1346115545, 11124.8607402743,
11128.5870140058, 11132.3134327067, 11136.0399963352, 11139.7667048493, 11143.4935582069, 11147.2205563661, 11150.9476992850, 11154.6749869217, 11158.4024192344, 11162.1299961811,
11165.8577177201, 11169.5855838095, 11173.3135944075, 11177.0417494725, 11180.7700489627, 11184.4984928364, 11188.2270810518, 11191.9558135674, 11195.6846903416, 11199.4137113326,
11203.1428764989, 11206.8721857990, 11210.6016391913, 11214.3312366343, 11218.0609780865, 11221.7908635065, 11225.5208928527, 11229.2510660839, 11232.9813831586, 11236.7118440354,
11240.4424486729, 11244.1731970299, 11247.9040890651, 11251.6351247372, 11255.3663040049, 11259.0976268270, 11262.8290931623, 11266.5607029696, 11270.2924562078, 11274.0243528358,
11277.7563928123, 11281.4885760964, 11285.2209026469, 11288.9533724229, 11292.6859853833, 11296.4187414872, 11300.1516406935, 11303.8846829613, 11307.6178682497, 11311.3511965178,
11315.0846677247, 11318.8182818296, 11322.5520387916, 11326.2859385700, 11330.0199811239, 11333.7541664126, 11337.4884943954, 11341.2229650315, 11344.9575782803, 11348.6923341011,
11352.4272324533, 11356.1622732961, 11359.8974565892, 11363.6327822917, 11367.3682503634, 11371.1038607635, 11374.8396134515, 11378.5755083871, 11382.3115455298, 11386.0477248391,
11389.7840462745, 11393.5205097958, 11397.2571153626, 11400.9938629344, 11404.7307524711, 11408.4677839322, 11412.2049572775, 11415.9422724669, 11419.6797294599, 11423.4173282166,
11427.1550686965, 11430.8929508597, 11434.6309746659, 11438.3691400750, 11442.1074470470, 11445.8458955418, 11449.5844855194, 11453.3232169396, 11457.0620897626, 11460.8011039483,
11464.5402594567, 11468.2795562480, 11472.0189942823, 11475.7585735196, 11479.4982939201, 11483.2381554439, 11486.9781580512, 11490.7183017023, 11494.4585863573, 11498.1990119765,
11501.9395785202, 11505.6802859486, 11509.4211342221, 11513.1621233010, 11516.9032531456, 11520.6445237164, 11524.3859349738, 11528.1274868781, 11531.8691793899, 11535.6110124695,
11539.3529860775, 11543.0951001745, 11546.8373547208, 11550.5797496772, 11554.3222850041, 11558.0649606622, 11561.8077766121, 11565.5507328145, 11569.2938292300, 11573.0370658193,
11576.7804425431, 11580.5239593622, 11584.2676162374, 11588.0114131293, 11591.7553499988, 11595.4994268067, 11599.2436435140, 11602.9880000813, 11606.7324964697, 11610.4771326400,
11614.2219085532, 11617.9668241702, 11621.7118794520, 11625.4570743597, 11629.2024088541, 11632.9478828964, 11636.6934964476, 11640.4392494689, 11644.1851419212, 11647.9311737658,
11651.6773449639, 11655.4236554764, 11659.1701052648, 11662.9166942901, 11666.6634225136, 11670.4102898967, 11674.1572964004, 11677.9044419862, 11681.6517266155, 11685.3991502494,
11689.1467128494, 11692.8944143769, 11696.6422547933, 11700.3902340600, 11704.1383521385, 11707.8866089902, 11711.6350045766, 11715.3835388593, 11719.1322117998, 11722.8810233597,
11726.6299735005, 11730.3790621838, 11734.1282893712, 11737.8776550245, 11741.6271591052, 11745.3768015751, 11749.1265823958, 11752.8765015291, 11756.6265589367, 11760.3767545805,
11764.1270884221, 11767.8775604235, 11771.6281705464, 11775.3789187527, 11779.1298050043, 11782.8808292630, 11786.6319914909, 11790.3832916498, 11794.1347297016, 11797.8863056085,
11801.6380193323, 11805.3898708351, 11809.1418600790, 11812.8939870260, 11816.6462516382, 11820.3986538777, 11824.1511937066, 11827.9038710871, 11831.6566859813, 11835.4096383515,
11839.1627281598, 11842.9159553685, 11846.6693199399, 11850.4228218362, 11854.1764610196, 11857.9302374526, 11861.6841510974, 11865.4382019165, 11869.1923898721, 11872.9467149267,
11876.7011770428, 11880.4557761826, 11884.2105123088, 11887.9653853837, 11891.7203953699, 11895.4755422299, 11899.2308259263, 11902.9862464216, 11906.7418036783, 11910.4974976592,
11914.2533283267, 11918.0092956436, 11921.7653995726, 11925.5216400762, 11929.2780171172, 11933.0345306584, 11936.7911806625, 11940.5479670922, 11944.3048899103, 11948.0619490797,
11951.8191445631, 11955.5764763235, 11959.3339443236, 11963.0915485264, 11966.8492888948, 11970.6071653917, 11974.3651779800, 11978.1233266228, 11981.8816112829, 11985.6400319235,
11989.3985885075, 11993.1572809980, 11996.9161093581, 12000.6750735509, 12004.4341735394, 12008.1934092867, 12011.9527807562, 12015.7122879108, 12019.4719307138, 12023.2317091285,
12026.9916231180, 12030.7516726455, 12034.5118576744, 12038.2721781680, 12042.0326340895, 12045.7932254023, 12049.5539520697, 12053.3148140551, 12057.0758113219, 12060.8369438335,
12064.5982115533, 12068.3596144448, 12072.1211524714, 12075.8828255966, 12079.6446337839, 12083.4065769969, 12087.1686551991, 12090.9308683541, 12094.6932164254, 12098.4556993767,
12102.2183171715, 12105.9810697736, 12109.7439571466, 12113.5069792541, 12117.2701360599, 12121.0334275277, 12124.7968536213, 12128.5604143043, 12132.3241095407, 12136.0879392941,
12139.8519035284, 12143.6160022074, 12147.3802352951, 12151.1446027552, 12154.9091045517, 12158.6737406485, 12162.4385110096, 12166.2034155988, 12169.9684543802, 12173.7336273177,
12177.4989343754, 12181.2643755173, 12185.0299507075, 12188.7956599100, 12192.5615030889, 12196.3274802083, 12200.0935912324, 12203.8598361253, 12207.6262148512, 12211.3927273743,
12215.1593736587, 12218.9261536687, 12222.6930673685, 12226.4601147225, 12230.2272956948, 12233.9946102498, 12237.7620583518, 12241.5296399652, 12245.2973550543, 12249.0652035835,
12252.8331855171, 12256.6013008196, 12260.3695494555, 12264.1379313891, 12267.9064465851, 12271.6750950077, 12275.4438766216, 12279.2127913914, 12282.9818392814, 12286.7510202564,
12290.5203342808, 12294.2897813194, 12298.0593613367, 12301.8290742973, 12305.5989201660, 12309.3688989073, 12313.1390104861, 12316.9092548670, 12320.6796320148, 12324.4501418941,
12328.2207844699, 12331.9915597068, 12335.7624675698, 12339.5335080235, 12343.3046810330, 12347.0759865629, 12350.8474245783, 12354.6189950441, 12358.3906979250, 12362.1625331862,
12365.9345007925, 12369.7066007090, 12373.4788329006, 12377.2511973323, 12381.0236939693, 12384.7963227764, 12388.5690837189, 12392.3419767617, 12396.1150018701, 12399.8881590091,
12403.6614481438, 12407.4348692395, 12411.2084222613, 12414.9821071745, 12418.7559239442, 12422.5298725356, 12426.3039529141, 12430.0781650449, 12433.8525088933, 12437.6269844245,
12441.4015916041, 12445.1763303972, 12448.9512007692, 12452.7262026856, 12456.5013361117, 12460.2766010130, 12464.0519973548, 12467.8275251027, 12471.6031842221, 12475.3789746785,
12479.1548964375, 12482.9309494644, 12486.7071337250, 12490.4834491848, 12494.2598958092, 12498.0364735641, 12501.8131824148, 12505.5900223272, 12509.3669932668, 12513.1440951994,
12516.9213280906, 12520.6986919061, 12524.4761866116, 12528.2538121730, 12532.0315685559, 12535.8094557261, 12539.5874736495, 12543.3656222918, 12547.1439016189, 12550.9223115967,
12554.7008521909, 12558.4795233676, 12562.2583250925, 12566.0372573317, 12569.8163200511, 12573.5955132165, 12577.3748367941, 12581.1542907498, 12584.9338750496, 12588.7135896595,
12592.4934345456, 12596.2734096739, 12600.0535150106, 12603.8337505217, 12607.6141161734, 12611.3946119317, 12615.1752377629, 12618.9559936331, 12622.7368795085, 12626.5178953553,
12630.2990411397, 12634.0803168280, 12637.8617223864, 12641.6432577813, 12645.4249229788, 12649.2067179453, 12652.9886426472, 12656.7706970508, 12660.5528811224, 12664.3351948284,
12668.1176381353, 12671.9002110094, 12675.6829134172, 12679.4657453251, 12683.2487066996, 12687.0317975072, 12690.8150177144, 12694.5983672877, 12698.3818461936, 12702.1654543987,
12705.9491918696, 12709.7330585728, 12713.5170544751, 12717.3011795429, 12721.0854337429, 12724.8698170418, 12728.6543294063, 12732.4389708030, 12736.2237411987, 12740.0086405602,
12743.7936688540, 12747.5788260470, 12751.3641121060, 12755.1495269978, 12758.9350706892, 12762.7207431470, 12766.5065443381, 12770.2924742293, 12774.0785327874, 12777.8647199795,
12781.6510357725, 12785.4374801331, 12789.2240530285, 12793.0107544256, 12796.7975842913, 12800.5845425926, 12804.3716292966, 12808.1588443704, 12811.9461877808, 12815.7336594951,
12819.5212594803, 12823.3089877036, 12827.0968441319, 12830.8848287325, 12834.6729414725, 12838.4611823191, 12842.2495512395, 12846.0380482008, 12849.8266731704, 12853.6154261153,
12857.4043070030, 12861.1933158006, 12864.9824524754, 12868.7717169948, 12872.5611093260, 12876.3506294364, 12880.1402772934, 12883.9300528643, 12887.7199561165, 12891.5099870174,
12895.3001455344, 12899.0904316350, 12902.8808452867, 12906.6713864568, 12910.4620551128, 12914.2528512224, 12918.0437747530, 12921.8348256721, 12925.6260039472, 12929.4173095461,
12933.2087424362, 12937.0003025851, 12940.7919899605, 12944.5838045299, 12948.3757462612, 12952.1678151218, 12955.9600110796, 12959.7523341021, 12963.5447841572, 12967.3373612125,
12971.1300652358, 12974.9228961949, 12978.7158540575, 12982.5089387915, 12986.3021503647, 12990.0954887449, 12993.8889538999, 12997.6825457976, 13001.4762644059, 13005.2701096927,
13009.0640816259, 13012.8581801734, 13016.6524053032, 13020.4467569833, 13024.2412351815, 13028.0358398660, 13031.8305710047, 13035.6254285656, 13039.4204125169, 13043.2155228265,
13047.0107594625, 13050.8061223930, 13054.6016115862, 13058.3972270101, 13062.1929686329, 13065.9888364227, 13069.7848303478, 13073.5809503762, 13077.3771964763, 13081.1735686162,
13084.9700667641, 13088.7666908884, 13092.5634409572, 13096.3603169390, 13100.1573188018, 13103.9544465142, 13107.7517000444, 13111.5490793607, 13115.3465844316, 13119.1442152254,
13122.9419717105, 13126.7398538553, 13130.5378616282, 13134.3359949977, 13138.1342539322, 13141.9326384003, 13145.7311483704, 13149.5297838110, 13153.3285446906, 13157.1274309778,
13160.9264426411, 13164.7255796491, 13168.5248419705, 13172.3242295736, 13176.1237424273, 13179.9233805001, 13183.7231437607, 13187.5230321778, 13191.3230457199, 13195.1231843559
      };

   if (x<0)
      mexErrMsgTxt("Gamma function of negative number\n");
   else if (x>=4000)
      return gsl_sf_lngamma(0.5*x);
   else
      return temp[x];
}
inline double my_sf_lngamma(double x)
{
   return my_sf_lngamma((int)(x+0.5));
}

// calculate logmgamma( x/2 );
inline double mylogmgamma(int x, int D)
{
   //double val = 0.25*D*(D-1)*log(pi);
   double val = 0.2861824715 * D * (D-1);
   switch (D)
   {
      case 1:
         val += my_sf_lngamma(x);
         break;
      case 2:
         val += my_sf_lngamma(x)+my_sf_lngamma(x-1);
         break;
      case 3:
         val += my_sf_lngamma(x)+my_sf_lngamma(x-1)+my_sf_lngamma(x-2);
         break;
      default:
         for (int j=1; j<=D; j++)
            val += my_sf_lngamma(x - j);
         break;
   }

   return val;
}

inline double mylogmgamma(double x, int D)
{
   return mylogmgamma((int)(x+0.5), D);
}

inline double mylogmgamma(int x, double D)
{
   return mylogmgamma(x, (int)(D+0.5));
}

inline double mylogmgamma(double x, double D)
{
   return mylogmgamma((int)(x+0.5), (int)(D+0.5));
}

/*inline double mgamma(double x, double D)
{
   double val = pow(pi, 0.25*D*(D-1));
   for (int j=0; j<=D; j++)
      val *= gsl_sf_gamma(x + (1-j)/2);
   return val;
}
inline double logmgamma(double x, int D)
{
   return mylogmgamma((int)(x*2 + 0.5), D);

   //double val = 0.25*D*(D-1)*log(pi);
   double val = 0.2861824715 * D * (D-1);
   switch (D)
   {
      case 1:
         val += gsl_sf_lngamma(x);
         break;
      case 2:
         val += gsl_sf_lngamma(x)+gsl_sf_lngamma(x-0.5);
         break;
      case 3:
         val += gsl_sf_lngamma(x)+gsl_sf_lngamma(x-0.5)+gsl_sf_lngamma(x-1);
         break;
      default:
         for (int j=1; j<=D; j++)
            val += gsl_sf_lngamma(x + (1-j)/2);
         break;
   }

   return val;
}*/



#endif
